CREATE TEMPORARY SEQUENCE tuple_id_seq;

-- C(artists -> artists.name)
WITH relationship AS (
    SELECT artists.tuple_id AS key, artists.name AS val
    FROM (
        SELECT nextval('tuple_id_seq') AS tuple_id, name
        FROM artists
        ) AS artists
),
cardlist AS (
	SELECT count(val) AS card -- NO DISTINCT IF COUNTING TUPLES!!!
	FROM relationship
	GROUP BY key
)
SELECT card, count(*)
FROM cardlist
GROUP BY card
ORDER BY card ASC;

-- C(artist -> atrist_aliases)
WITH relationship AS (
	SELECT artists.tuple_id AS key, artists_aliases.artist_id AS val
	FROM (
		SELECT nextval('tuple_id_seq') AS tuple_id, id
		FROM artists
		) AS artists
	LEFT OUTER JOIN artists_aliases ON artists.id = artists_aliases.artist_id
),
cardlist AS (
	SELECT count(val) AS card -- NO DISTINCT IF COUNTING TUPLES!!!
	FROM relationship
	GROUP BY key
)
SELECT card, count(*)
FROM cardlist
GROUP BY card
ORDER BY card ASC

-- C(artist -> atrist_aliases.alias_id)
WITH relationship AS (
	SELECT artists.tuple_id AS key, artists_aliases.artist_id AS val
	FROM (
		SELECT nextval('tuple_id_seq') AS tuple_id, id
		FROM artists
		) AS artists
	LEFT OUTER JOIN (
		SELECT DISTINCT artist_id
		FROM artists_aliases
		) AS artists_aliases ON artists.id = artists_aliases.artist_id
),
cardlist AS (
	SELECT count(distinct val) AS card
	FROM relationship
	GROUP BY key
)
SELECT card, count(*)
FROM cardlist
GROUP BY card
ORDER BY card ASC

-- C(artist -> atrist_aliases.name)
WITH relationship AS (
	SELECT artists.tuple_id AS key, artists_aliases.name AS val
	FROM (
		SELECT nextval('tuple_id_seq') AS tuple_id, id
		FROM artists
		) AS artists
	LEFT OUTER JOIN (
		SELECT DISTINCT artist_id, name
		FROM artists_aliases
		) AS artists_aliases ON artists.id = artists_aliases.artist_id
),
cardlist AS (
	SELECT count(distinct val) AS card
	FROM relationship
	GROUP BY key
)
SELECT card, count(*)
FROM cardlist
GROUP BY card
ORDER BY card ASC

-- C(artist.id -> atrist_aliases)
WITH relationship AS (
	SELECT artists.id AS key, artists_aliases.artist_id AS val
	FROM (
		SELECT DISTINCT id
		FROM artists
		WHERE id IS NOT NULL
		) AS artists
	LEFT OUTER JOIN artists_aliases ON artists.id = artists_aliases.artist_id
),
cardlist AS (
	SELECT count(val) AS card -- NO DISTINCT IF COUNTING TUPLES!!!
	FROM relationship
	GROUP BY key
)
SELECT card, count(*)
FROM cardlist
GROUP BY card
ORDER BY card ASC

-- C(artist.id -> artist_aliases.artist_id)
WITH relationship AS (
	SELECT artists.id AS key, artists_aliases.artist_id AS val
	FROM (
		SELECT DISTINCT id
		FROM artists
		WHERE id IS NOT NULL
		) AS artists
	LEFT OUTER JOIN artists_aliases ON artists.id = artists_aliases.artist_id
),
cardlist AS (
	SELECT count(distinct val) AS card
	FROM relationship
	GROUP BY key
)
SELECT card, count(*)
FROM cardlist
GROUP BY card
ORDER BY card ASC

-- C(artist.id -> atrist_aliases.name)
WITH relationship AS (
	SELECT artists.id AS key, artists_aliases.name AS val
	FROM (
		SELECT DISTINCT id
		FROM artists
		WHERE id IS NOT NULL
		) AS artists
	LEFT OUTER JOIN (
		SELECT DISTINCT artist_id, name
		FROM artists_aliases
		) AS artists_aliases ON artists.id = artists_aliases.artist_id
),
cardlist AS (
	SELECT count(distinct val) AS card
	FROM relationship
	GROUP BY key
)
SELECT card, count(*)
FROM cardlist
GROUP BY card
ORDER BY card ASC

-- C(artist.realname -> artist_aliases)
WITH relationship AS (
	SELECT artists.realname AS key, artists_aliases.artist_id AS val
	FROM (
		SELECT DISTINCT realname, id
		FROM artists
		WHERE realname IS NOT NULL
		) AS artists
	LEFT OUTER JOIN artists_aliases ON artists.id = artists_aliases.artist_id
),
cardlist AS (
	SELECT count(val) AS card -- NO DISTINCT IF COUNTING TUPLES!!!
	FROM relationship
	GROUP BY key
)
SELECT card, count(*)
FROM cardlist
GROUP BY card
ORDER BY card ASC

-- C(artist.realname -> atrist_aliases.id)
WITH relationship AS (
	SELECT artists.realname AS key, artists_aliases.artist_id AS val
	FROM (
		SELECT DISTINCT realname, id
		FROM artists
		WHERE realname IS NOT NULL
		) AS artists
	LEFT OUTER JOIN artists_aliases ON artists.id = artists_aliases.artist_id
),
cardlist AS (
	SELECT count(distinct val) AS card
	FROM relationship
	GROUP BY key
)
SELECT card, count(*)
FROM cardlist
GROUP BY card
ORDER BY card ASC

-- C(artist.realname -> atrist_aliases.name)
WITH relationship AS (
	SELECT artists.id AS key, artists_aliases.name AS val
	FROM (
		SELECT DISTINCT realname, id
		FROM artists
		WHERE realname IS NOT NULL
		) AS artists
	LEFT OUTER JOIN (
		SELECT DISTINCT artist_id, name
		FROM artists_aliases
		) AS artists_aliases ON artists.id = artists_aliases.artist_id
),
cardlist AS (
	SELECT count(distinct val) AS card
	FROM relationship
	GROUP BY key
)
SELECT card, count(*)
FROM cardlist
GROUP BY card
ORDER BY card ASC

-- C(artist -> artist.realname UNION artist -> artist.name)
WITH fork AS (
    SELECT nextval('tuple_id_seq') AS tuple_id, realname, name
	FROM artists
),
relationship AS (
	SELECT key, val
	FROM (
		SELECT DISTINCT tuple_id AS key, realname AS val
		FROM fork
	) AS path1
	UNION (
	SELECT DISTINCT tuple_id AS key, name AS val
		FROM fork
	)
),
cardlist AS (
	SELECT count(distinct val) AS card
	FROM relationship
	GROUP BY key
)
SELECT card, count(*)
FROM cardlist
GROUP BY card
ORDER BY card ASC



-- C(artist.id -> artist.realname UNION artist -> artist.name)
WITH fork AS (
    SELECT id, realname, name
	FROM artists
),
relationship AS (
	SELECT key, val
	FROM (
		SELECT DISTINCT id AS key, realname AS val
		FROM fork
	) AS path1
	UNION (
	SELECT DISTINCT id AS key, name AS val
		FROM fork
	)
),
cardlist AS (
	SELECT count(distinct val) AS card
	FROM relationship
	GROUP BY key
)
SELECT card, count(*)
FROM cardlist
GROUP BY card
ORDER BY card ASC

-- Alternatively no fork necessary when not beginning with a tuple
WITH relationship AS (
	SELECT key, val
	FROM (
		SELECT DISTINCT id AS key, realname AS val
		FROM artists
	) AS path1
	UNION (
	SELECT DISTINCT id AS key, name AS val
		FROM artists
	)
),
cardlist AS (
	SELECT count(distinct val) AS card
	FROM relationship
	GROUP BY key
)
SELECT card, count(*)
FROM cardlist
GROUP BY card
ORDER BY card ASC

-- C(artists -> artists_aliases.name UNION artists -> artists_namevariations.name)
WITH fork1 AS (
	SELECT nextval('tuple_id_seq') AS tuple_id, id
	FROM artists
),
path1 AS (
	SELECT DISTINCT tuple_id AS key, artists_aliases.name AS val
	FROM fork1
	LEFT OUTER JOIN artists_aliases
		ON fork1.id = artists_aliases.artist_id
),
path2 AS (
	SELECT DISTINCT tuple_id AS key, artists_namevariations.name AS val
	FROM fork1
	LEFT OUTER JOIN artists_namevariations
		ON fork1.id = artists_namevariations.artist_id
),
relationship AS (
	SELECT key, val
	FROM (
		SELECT key, val
		FROM path1) AS path1
	UNION ( -- MUST USE UNION ALL WHEN GOING FOR TUPLES!!
		SELECT key, val
		FROM path2)
),
cardlist AS (
	SELECT count(distinct val) AS card
	FROM relationship
	GROUP BY key
)
SELECT card, count(*)
FROM cardlist
GROUP BY card
ORDER BY card ASC

-- C(releases_tracklist_artists.track_id -> releases_tracklist_artists JOIN releases_tracklist_artists.id -> releases_tracklist_artists)
WITH coalesce AS (
	SELECT nextval('tuple_id_seq') AS tuple_id, track_id, id
	FROM releases_tracklist_artists
),
path1 AS (
	SELECT DISTINCT track_id AS key, tuple_id AS val
	FROM coalesce
	WHERE track_id IS NOT NULL
),
path2 AS (
	SELECT DISTINCT id AS key, tuple_id AS val
	FROM coalesce
	WHERE track_id IS NOT NULL
),
relationship AS (
	SELECT path1.key AS key1, path2.key AS key2, path1.val AS val
	FROM path1 JOIN path2 ON path1.val = path2.val
),
cardlist AS (
	SELECT count(distinct val) AS card
	FROM relationship
	GROUP BY key1, key2
)
SELECT card, count(*)
FROM cardlist
GROUP BY card
ORDER BY card ASC

-- Alternative straight-forward version (relation already contains the join)
WITH relationship AS (
	SELECT id AS key1, track_id AS key2
	FROM releases_tracklist_artists
	WHERE id IS NOT NULL AND track_id IS NOT NULL
	-- Beware: Join only "distincted" relations!
),
cardlist AS (
	SELECT count(*) AS card
	FROM relationship
	GROUP BY key1, key2
)
SELECT card, count(*)
FROM cardlist
GROUP BY card
ORDER BY card ASC

-- JOIN with equality coalesce
-- C(releases_tracklist_artists.name -> releases_tracklist JOIN releases_tracklist_extraartist.name -> releases_tracklist)
WITH coalesce AS (
	SELECT nextval('tuple_id_seq') AS tuple_id, track_id
	FROM releases_tracklist
),
path1 AS (
	SELECT DISTINCT releases_tracklist_artists.name AS key, tuple_id AS val
	FROM coalesce
	JOIN releases_tracklist_artists
		ON coalesce.track_id = releases_tracklist_artists.track_id
	WHERE releases_tracklist_artists.name IS NOT NULL
),
path2 AS (
	SELECT DISTINCT releases_tracklist_extraartists.name AS key, tuple_id AS val
	FROM coalesce
	JOIN releases_tracklist_extraartists
		ON coalesce.track_id = releases_tracklist_extraartists.track_id
	WHERE releases_tracklist_extraartists.name IS NOT NULL
),
relationship AS (
	SELECT path1.key AS key1, path2.key AS key2, path1.val AS val
	FROM path1 JOIN path2 ON path1.val = path2.val
	order by key1, key2
),
cardlist AS (
	SELECT count(distinct val) AS card
	FROM relationship
	GROUP BY key1, key2
)
--select * from relationship
SELECT card, count(*)
FROM cardlist
GROUP BY card
ORDER BY card ASC

-- Alternative straight-forward version (relation already contains the join)
WITH relationship AS (
	SELECT releases_tracklist.track_id, releases_tracklist_artists.name AS key1, releases_tracklist_extraartists.name AS key2
	FROM releases_tracklist
	JOIN (SELECT DISTINCT track_id, name FROM releases_tracklist_artists) AS releases_tracklist_artists
		ON releases_tracklist.track_id = releases_tracklist_artists.track_id
	JOIN (SELECT DISTINCT track_id, name FROM releases_tracklist_extraartists) AS releases_tracklist_extraartists
		ON releases_tracklist.track_id = releases_tracklist_extraartists.track_id
	WHERE releases_tracklist_artists.name IS NOT NULL 
	AND releases_tracklist_extraartists.name IS NOT NULL
	ORDER BY releases_tracklist.track_id
),
cardlist AS (
	SELECT count(*) AS card
	FROM relationship
	GROUP BY key1, key2
)
SELECT card, count(*)
FROM cardlist
GROUP BY card
ORDER BY card ASC

-- test FD on releases: country, title -> released
-- C(((releases.country -> releases) JOIN (releases.title -> releases) o (releases -> releases.released))
-- directly take the straight-forward way
WITH junction AS (
	SELECT country, title, released
	FROM releases
),
relationship AS (
	SELECT junction.country AS key1, junction.title AS key2, junction.released AS val
	FROM junction
	WHERE junction.country IS NOT NULL 
	AND junction.released IS NOT NULL
	
),
cardlist AS (
	SELECT count(distinct val) AS card
	FROM relationship
	GROUP BY key1, key2
)
SELECT card, count(*)
FROM cardlist
GROUP BY card
ORDER BY card ASC