package de.uni_potsdam.hpi.isg.efes.csg.datafit;

import java.io.PrintStream;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

public class DataFitReport {

	private static Logger logger = LogManager.getLogger(DataFitReport.class.getSimpleName());

	private Map<Class<? extends Section>, Section> sections = new HashMap<Class<? extends Section>, DataFitReport.Section>();

	public void add(Section section) {
		if (section == null) {
			throw new IllegalArgumentException();
		}

		if (this.sections.containsKey(section.getClass())) {
			logger.warn("There already is a section of type {}, it will be replaced.",
					section.getClass());
		}

		this.sections.put(section.getClass(), section);
	}

	public Map<Class<? extends Section>, Section> getSections() {
		return Collections.unmodifiableMap(this.sections);
	}

	public static interface Section {

		void print(PrintStream printStream);
		
		void printCsv(PrintStream printStream);
	}

	public void print(PrintStream stream) {
		
		for (Section section : this.sections.values()) {
			section.print(stream);
			stream.println();
		}
		
	}
	
}
