package de.uni_potsdam.hpi.isg.efes.csg.datafit.simple;

import java.util.List;

import de.uni_potsdam.hpi.isg.efes.csg.datafit.DataFitModel;
import de.uni_potsdam.hpi.isg.efes.csg.datafit.DataFitReport;
import de.uni_potsdam.hpi.isg.efes.csg.mapping.NodeCorrespondence;
import de.uni_potsdam.hpi.isg.efes.csg.mapping.NodeMapping;
import de.uni_potsdam.hpi.isg.efes.csg.nodes.Node;
import de.uni_potsdam.hpi.isg.efes.csg.scenario.DataSet;
import de.uni_potsdam.hpi.isg.efes.csg.scenario.IntegrationScenario;

public class SimpleComplexityModel implements DataFitModel {

	private IntegrationScenario integrationScenario;

	@Override
	public void setIntegrationScenario(IntegrationScenario scenario) {
		this.integrationScenario = scenario;
	}

	@Override
	public void evaluateScenarioTo(DataFitReport report) {
		if (this.integrationScenario == null) {
			throw new IllegalStateException();
		}

		SimpleReportSection section = new SimpleReportSection();

		DataSet target = this.integrationScenario.getTargetDataSet();
		SimpleReportSection.DataSet targetReport = new SimpleReportSection.DataSet(target.getSchema().getName());
		countSchemaElements(target, targetReport);
		int numTargetCorrespondences = 0;

		for (DataSet source : this.integrationScenario.getSourceDataSets()) {
			SimpleReportSection.DataSet dataSetReport = new SimpleReportSection.DataSet(source.getSchema().getName());
			countSchemaElements(source, dataSetReport);

			int numSourceCorrespondences = 0;
			NodeMapping mapping = this.integrationScenario.getMappings().get(source);
			for (List<NodeCorrespondence> correspondences : mapping.getNodeCorrespondences().values()) {
				for (NodeCorrespondence correspondence : correspondences) {
					if (!correspondence.getTargetNode().isMetaData()) {
						numSourceCorrespondences++;
						numTargetCorrespondences++;
					}
				}
			}
			dataSetReport.setNumCorrespondences(numSourceCorrespondences);

			section.addSourceDataSet(dataSetReport);
		}

		targetReport.setNumCorrespondences(numTargetCorrespondences);
		section.setTargetDataSet(targetReport);

		report.add(section);
	}

	private void countSchemaElements(DataSet source, SimpleReportSection.DataSet dataSetReport) {
		int numMetaDataAttributes = 0;
		int numAttributes = 0;
		int numTables = 0;
		for (Node node : source.getSchema().getAtomicNodes().values()) {
			if (node.isSimpleIdentityNode()) {
				numTables++;
			} else if (node.isSimpleValueNode()) {
				numAttributes++;
				if (node.isMetaData()) {
					numMetaDataAttributes++;
				}
			}
		}
		dataSetReport.setNumTables(numTables);
		dataSetReport.setNumAttributes(numAttributes);
		dataSetReport.setNumMetaDataAttributes(numMetaDataAttributes);
	}

}
