package de.uni_potsdam.hpi.isg.efes.csg.datafit.values.data_access;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

public class String2IntValueRetriever extends ColumnIterator.DefaultValueRetriever {

	private static final Logger logger = LogManager.getLogger(String2IntValueRetriever.class.getSimpleName());
	
	public static final String2IntValueRetriever INSTANCE = new String2IntValueRetriever();
	
	
	/**
	 * If the retrieved string contains exactly one number, it is considered a number. This should cover the following scenarios:
	 * <ul>
	 * <li>123</li>
	 * <li>-123</li>
	 * <li>$ 123</li>
	 * <li>123 cm</li>
	 * </ul>
	 * However, ambiguous numbers like <i>2012-01-02</i> are not accepted.
	 */
	private static final Pattern CONTAINED_NUMBER_PATTERN = Pattern.compile("\\D*([\\+-]?\\d+)\\D*");
	
	@Override
	public int getInt(ResultSet resultSet) throws SQLException {
		String str = getString(resultSet);
		if (str == null) {
			setRetrieveSuccess(true);
			logger.trace("Converting {} to {}.", str, 0);
			return 0;
		}
		
		Matcher matcher = CONTAINED_NUMBER_PATTERN.matcher(str);
		if (matcher.matches()) {
			String numberString = matcher.group(1);
			int result = Integer.valueOf(numberString);
			setRetrieveSuccess(true);
			logger.trace("Converting {} to {}.", str, result);
			return result;
		}
		
		setRetrieveSuccess(false);
		logger.trace("Cannot convert {}.", str);
		return 0;
	}
	
	@Override
	public boolean isValueValid(ResultSet resultSet) throws SQLException {
		getInt(resultSet);
		return couldRetrieveValidValue();
	}

}
