package de.uni_potsdam.hpi.isg.efes.csg.datafit.values.metrics;

import de.uni_potsdam.hpi.isg.efes.csg.nodes.Datatype.GeneralType;

public abstract class AbstractSymmetricValueFitMetric extends AbstractValueFitMetric {

	public AbstractSymmetricValueFitMetric(GeneralType... supportedTypes) {
		super(supportedTypes);
	}

	@Override
	public SourceOperator createSourceOperator() {
		return createOperator();
	}

	@Override
	public TargetOperator createTargetOperator() {
		return createOperator();
	}

	abstract protected SymmetricOperator createOperator();

	public static abstract class SymmetricOperator implements SourceOperator, TargetOperator {

		@Override
		public SourceColumnStatistic concludeSourceResult() {
			return concludeResult();
		}

		@Override
		public TargetColumnStatistic concludeTargetResult() {
			return concludeResult();
		}

		abstract protected SymmetricColumnStatistic concludeResult();

	}

	public abstract static class SymmetricColumnStatistic extends DefaultColumnStatistic implements
			SourceColumnStatistic, TargetColumnStatistic {

		private double relevance;
		
		public SymmetricColumnStatistic(Class<? extends ValueFitMetric> metricClass, double relevance) {
			super(metricClass);
			this.relevance = relevance;
		}
		@Override
		public Result compare(SourceColumnStatistic otherResult) {
			double fitValue = calculateFitValue(otherResult);
			return new DefaultResult(fitValue, getRelevance(), getMetricClass());
		}
		
		@Override
		public double getRelevance() {
			return this.relevance;
		}
		
		abstract protected double calculateFitValue(SourceColumnStatistic otherResult);

	}

}
