package de.uni_potsdam.hpi.isg.efes.csg.datafit.values.metrics;

import java.sql.SQLException;

import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.data_access.ColumnIterator;
import de.uni_potsdam.hpi.isg.efes.csg.nodes.Datatype.GeneralType;

public class ConstancyMetric extends AbstractSymmetricValueFitMetric {

	@Override
	protected Operator createOperator() {
		return new Operator();
	}
	
	@Override
	public boolean isApplicableFor(GeneralType type) {
		return true;
	}
	
	public static class Operator extends SymmetricOperator {

		private int numValues;
		private double unnormalizedConstancy;

		@Override
		public void update(ColumnIterator columnIterator) throws SQLException {
			if (!columnIterator.wasNull() && columnIterator.getValueAsString() != null) {
				int count = columnIterator.getValueCount();
				numValues += count;
				unnormalizedConstancy += count * Math.log(count);
			}
		}

		@Override
		public ColumnStatistic concludeResult() {
			if (numValues == 0 || numValues == 1)
				return new ColumnStatistic(0.5);
			return new ColumnStatistic(unnormalizedConstancy / (numValues * Math.log(numValues)));
		}

	}

	public static class ColumnStatistic extends SymmetricColumnStatistic {

		private double constancy;
		
		public ColumnStatistic(double constancyResult) {
			super(ConstancyMetric.class, 1d);
			this.constancy = constancyResult;
		}

		@Override
		public double calculateFitValue(ValueFitMetric.SourceColumnStatistic otherResult) {
			if (!(otherResult instanceof ColumnStatistic)) {
				throw new IllegalArgumentException("Cannot compare constancy with " + otherResult + ".");
			}
			ColumnStatistic other = (ColumnStatistic) otherResult;
			return 1 - Math.abs(this.constancy - other.constancy);
		}
		
		public double getConstancy() {
			return constancy;
		}

		@Override
		public String toString() {
			return "Result [constancy=" + constancy + "]";
		}

	}

}
