package de.uni_potsdam.hpi.isg.efes.csg.datafit.values.metrics;

import it.unimi.dsi.fastutil.ints.Int2DoubleAVLTreeMap;
import it.unimi.dsi.fastutil.ints.Int2DoubleMap;
import it.unimi.dsi.fastutil.ints.Int2IntAVLTreeMap;
import it.unimi.dsi.fastutil.ints.Int2IntMap;

import java.sql.SQLException;

import org.apache.logging.log4j.LogManager;

import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.data_access.ColumnIterator;
import de.uni_potsdam.hpi.isg.efes.csg.nodes.Datatype;

public class ExponentialHistogramMetric extends AbstractSymmetricValueFitMetric {

	public ExponentialHistogramMetric() {
		super(Datatype.GeneralType.INTEGER);
	}

	@Override
	protected Operator createOperator() {
		return new Operator();
	}
	
	public static class Operator extends SymmetricOperator {

		private Int2IntMap histogram = new Int2IntAVLTreeMap();
		private int sum = 0;

		@Override
		public void update(ColumnIterator columnIterator) throws SQLException {
			int count = columnIterator.getValueCount();
			int value = columnIterator.getValueAsInt();

			if (!columnIterator.wasNull() && columnIterator.wasValid()) {
				int key = calculateHistogramKey(value);
				int oldCount = this.histogram.get(key);
				this.histogram.put(key, oldCount + count);
				sum += count;
			}

		}

		private int calculateHistogramKey(int value) {
			if (value >= 0) {
				int leadingZeros = Integer.numberOfLeadingZeros(value);
				return Integer.SIZE - leadingZeros;

			} else {
				int leadingZeros = Integer.numberOfLeadingZeros(-value);
				return leadingZeros - Integer.SIZE;
			}
		}

		@Override
		public ColumnStatistic concludeResult() {
			Int2DoubleMap frequencies = new Int2DoubleAVLTreeMap();
			double squareSum = 0;
			for (Int2IntMap.Entry histogramEntry : this.histogram.int2IntEntrySet()) {
				double frequency = histogramEntry.getIntValue() / (double) this.sum;
				frequencies.put(histogramEntry.getIntKey(), frequency);
				squareSum += frequency * frequency;
			}
			double relevance = 0;
			if (!frequencies.isEmpty()) {
				relevance = Math.sqrt(squareSum / frequencies.size());
			}
			// Boost relevance and normalize by a set of 2 entries.
			relevance = correctFitValue(relevance * 2);
			LogManager.getRootLogger().debug("Entries: {}, relevance: {}.", frequencies.size(), relevance);
			return new ColumnStatistic(frequencies, relevance);
		}

	}

	public static class ColumnStatistic extends SymmetricColumnStatistic {

		private Int2DoubleMap frequencies;

		public ColumnStatistic(Int2DoubleMap frequencies, double relevance) {
			super(ExponentialHistogramMetric.class, relevance);
			this.frequencies = frequencies;
		}

		@Override
		public double calculateFitValue(ValueFitMetric.SourceColumnStatistic otherResult) {
			if (!(otherResult instanceof ColumnStatistic)) {
				throw new IllegalArgumentException("Cannot compare frequencies with " + otherResult + ".");
			}
			ColumnStatistic other = (ColumnStatistic) otherResult;
			if (this.frequencies.isEmpty() && other.frequencies.isEmpty()) {
				return 1;
			}
			
			double intersect = 0;
			for (Int2DoubleMap.Entry entry : this.frequencies.int2DoubleEntrySet()) {
				int key = entry.getIntKey();
				double thisFrequency = entry.getDoubleValue();
				double otherFrequency = other.frequencies.get(key);
				intersect += Math.min(thisFrequency, otherFrequency);
			}
			return intersect;
		}

		@Override
		public String toString() {
			return "Result [" + frequencies + "]";
		}

	}

}
