package de.uni_potsdam.hpi.isg.efes.csg.datafit.values.metrics;

import java.sql.SQLException;

import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.data_access.ColumnIterator;
import de.uni_potsdam.hpi.isg.efes.csg.nodes.Datatype.GeneralType;

public class FillStatusMetric extends AbstractSymmetricValueFitMetric {

	@Override
	protected Operator createOperator() {
		return new Operator();
	}
	
	@Override
	public boolean isApplicableFor(GeneralType type) {
		return true;
	}
	
	public static class Operator extends SymmetricOperator {

		public int numNulls, columnSize, numDistinctConvertibleValues, numInconvertibleValues, numDistinctInconvertibleValues;

		@Override
		public void update(ColumnIterator columnIterator) throws SQLException {
			String value = columnIterator.getValueAsString();
			int count = columnIterator.getValueCount();
			columnSize += count;

			if (!columnIterator.isValid()) {
				numDistinctInconvertibleValues++;
				numInconvertibleValues += count;
			
			} else if (value == null || value.isEmpty()) {
				numNulls += count;
			
			} else {
				numDistinctConvertibleValues++;
			}

		}

		@Override
		public ColumnStatistic concludeResult() {
			return new ColumnStatistic(this.numNulls, this.numInconvertibleValues, this.numDistinctInconvertibleValues, this.columnSize, this.numDistinctConvertibleValues);
		}

	}

	public static class ColumnStatistic extends SymmetricColumnStatistic {

		private double fillStatus;
		private int numNullValues;
		private int columnSize;
		private int numInconvertibleValues;
		private int numDistinctIncovertibleValues;
		private int numDistinctConvertibleValues;
		
		public ColumnStatistic(int numNullValues, int numInconvertibleValues, int numDistinctInconvertibleValues, int columnSize, int numDistinctConvertibleValues) {
			super(FillStatusMetric.class, 1d);
			this.fillStatus = columnSize == 0 ? 0 : (columnSize - numNullValues) / (double) columnSize;
			this.numNullValues = numNullValues;
			this.numInconvertibleValues = numInconvertibleValues;
			this.numDistinctIncovertibleValues = numDistinctInconvertibleValues;
			this.numDistinctConvertibleValues = numDistinctConvertibleValues;
			this.columnSize = columnSize;
		}

		@Override
		public double calculateFitValue(ValueFitMetric.SourceColumnStatistic otherResult) {
			if (!(otherResult instanceof ColumnStatistic)) {
				throw new IllegalArgumentException("Cannot compare fill status result with " + otherResult + ".");
			}
			ColumnStatistic other = (ColumnStatistic) otherResult;
			return 1 - Math.abs(this.fillStatus - other.fillStatus);
		}

		public double getFillStatus() {
			return this.fillStatus;
		}
		
		public int getColumnSize() {
			return columnSize;
		}

		public int getNumDistinctConvertibleValues() {
			return numDistinctConvertibleValues;
		}
		
		public int getNumConvertibleValues() {
			return columnSize - numNullValues - numInconvertibleValues;
		}
		
		public int getNumNullValues() {
			return numNullValues;
		}
		
		public int getNumInconvertibleValues() {
			return numInconvertibleValues;
		}
		
		public int getNumDistinctIncovertibleValues() {
			return numDistinctIncovertibleValues;
		}

		@Override
		public String toString() {
			return "Result [fillStatus=" + fillStatus + "]";
		}

	}

}
