package de.uni_potsdam.hpi.isg.efes.csg.datafit.values.metrics;

import java.math.BigInteger;
import java.sql.SQLException;

import org.apache.logging.log4j.LogManager;

import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.data_access.ColumnIterator;
import de.uni_potsdam.hpi.isg.efes.csg.nodes.Datatype;

public class MeanValueMetric extends AbstractSymmetricValueFitMetric {

	public MeanValueMetric() {
		super(Datatype.GeneralType.INTEGER);
	}

	@Override
	protected Operator createOperator() {
		return new Operator();
	}
	
	public static class Operator extends SymmetricOperator {

		private long numValues;
		private BigInteger sum = BigInteger.ZERO, squareLength = BigInteger.ZERO;

		@Override
		public void update(ColumnIterator columnIterator) throws SQLException {
			int count = columnIterator.getValueCount();
			int value = columnIterator.getValueAsInt();

			if (!columnIterator.wasNull() && columnIterator.wasValid()) {
				sum = sum.add(BigInteger.valueOf(value * count));
				numValues += count;
				squareLength = squareLength.add(BigInteger.valueOf(value).multiply(BigInteger.valueOf(value).multiply(BigInteger.valueOf(count))));
			}
		}

		@Override
		public ColumnStatistic concludeResult() {
			double mean;
			double variance;
			double relevance = 0;
			if (this.numValues == 0) {
				mean = Double.NaN;
				variance = Double.NaN;
				relevance = 0;
			} else {
				double sum = this.sum.doubleValue();
				double squareLength = this.squareLength.doubleValue();
				mean = sum / (double) numValues;
				variance = squareLength / (double) numValues - mean * mean;
				if (mean == 0) {
					relevance = 0;
				} else {
					double stDev = Math.sqrt(variance);
					relevance = Math.sqrt(correctFitValue(1 - (stDev / mean)));
					LogManager.getRootLogger().debug("Mean: {}, Standard Deviation: {}, Relevance: {}.",
							mean, stDev, relevance);
				}
			}
			return new ColumnStatistic(mean, variance, relevance);
		}

	}

	public static class ColumnStatistic extends SymmetricColumnStatistic {

		private double mean;
		private double stdDev;
		
		public ColumnStatistic(double mean, double variance, double relevance) {
			super(MeanValueMetric.class, relevance);
			this.mean = mean;
			this.stdDev = Math.sqrt(variance);
		}

		@Override
		public double calculateFitValue(ValueFitMetric.SourceColumnStatistic otherResult) {
			if (!(otherResult instanceof ColumnStatistic)) {
				throw new IllegalArgumentException("Cannot compare average string length result with " + otherResult + ".");
			}
			ColumnStatistic other = (ColumnStatistic) otherResult;
			double maxLength = Math.max(this.mean, other.mean);
			return correctFitValue(maxLength == 0 ? 1 :  1 - (Math.abs(this.mean - other.mean) / (2 * this.stdDev)));
		}

		@Override
		public String toString() {
			return "Result [mean=" + mean + "]";
		}

	}

}
