package de.uni_potsdam.hpi.isg.efes.csg.datafit.values.metrics;

import java.sql.SQLException;

import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.data_access.ColumnIterator;
import de.uni_potsdam.hpi.isg.efes.csg.nodes.Datatype;

public class RangeOverlapMetric extends AbstractSymmetricValueFitMetric {

	public RangeOverlapMetric() {
		super(Datatype.GeneralType.INTEGER);
	}

	@Override
	protected Operator createOperator() {
		return new Operator();
	}
	
	public static class Operator extends SymmetricOperator {

		private int minValue, maxValue;
		private boolean areValuesSeen = false;

		@Override
		public void update(ColumnIterator columnIterator) throws SQLException {
			int value = columnIterator.getValueAsInt();

			if (!columnIterator.wasNull() && columnIterator.wasValid()) {
				if (!this.areValuesSeen) {
					this.minValue = this.maxValue = value;
					this.areValuesSeen = true;
				} else {
					this.minValue = Math.min(value, this.minValue);
					this.maxValue = Math.max(value, this.maxValue);
				}
			}

		}

		@Override
		public ColumnStatistic concludeResult() {
			double relevance = 0;
			if (this.areValuesSeen) {
				int absMax = Math.max(Math.abs(this.minValue), Math.abs(this.maxValue));
				int maxDiff = Math.abs(Math.abs(this.minValue) - Math.abs(this.maxValue));
				if (absMax == 0) {
					relevance = 1;
				} else {
					relevance = (absMax - maxDiff) / absMax;
				}
			}
			return new ColumnStatistic(this.minValue, this.maxValue, this.areValuesSeen, relevance);
		}

	}

	public static class ColumnStatistic extends SymmetricColumnStatistic {

		private int minValue;
		private int maxValue;
		private boolean areValuesSeen;
		
		public ColumnStatistic(int minValue, int maxValue, boolean areValuesSeen, double relevance) {
			super(RangeOverlapMetric.class, relevance);
			this.minValue = minValue;
			this.maxValue = maxValue;
			this.areValuesSeen = areValuesSeen;
		}

		@Override
		public double calculateFitValue(ValueFitMetric.SourceColumnStatistic otherResult) {
			if (!(otherResult instanceof ColumnStatistic)) {
				throw new IllegalArgumentException("Cannot compare intervals with " + otherResult + ".");
			}
			ColumnStatistic other = (ColumnStatistic) otherResult;
			if (!this.areValuesSeen || !other.areValuesSeen) {
				return 0.5;
			}
			
			int outerMin = Math.min(this.minValue, other.minValue);
			int innerMin = Math.max(this.minValue, other.minValue);

			int outerMax = Math.max(this.maxValue, other.maxValue);
			int innerMax = Math.min(this.maxValue, other.maxValue);
			
			int unionSize = Math.max(0, outerMax - outerMin);
			int intersectionSize = Math.max(0, innerMax - innerMin);
			
			return unionSize == 0 ? 0 :  intersectionSize / (double) unionSize;
		}

		@Override
		public String toString() {
			return "Result [minValue=" + minValue + ", maxValue=" + maxValue + ", areValuesSeen=" + areValuesSeen + "]";
		}

	}

}
