package de.uni_potsdam.hpi.isg.efes.csg.datafit.values.metrics;

import java.sql.SQLException;

import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.data_access.ColumnIterator;
import de.uni_potsdam.hpi.isg.efes.csg.nodes.Datatype;

public interface ValueFitMetric {
	
	SourceOperator createSourceOperator();

	TargetOperator createTargetOperator();

	boolean isApplicableFor(Datatype.GeneralType type);
	
	interface SourceOperator {
		
		void update(ColumnIterator columnIterator) throws SQLException;
		
		SourceColumnStatistic concludeSourceResult();
		
	}
	
	interface MetricAssociate {

		Class<? extends ValueFitMetric> getMetricClass();
		
	}

	interface ColumnStatistic extends MetricAssociate {

	}
	
	interface SourceColumnStatistic extends ColumnStatistic {
	}
	
	interface TargetOperator {
		
		void update(ColumnIterator columnIterator) throws SQLException;
		
		TargetColumnStatistic concludeTargetResult();
	}

	
	
	interface TargetColumnStatistic extends ColumnStatistic {

		Result compare(SourceColumnStatistic otherResult);
		
		double getRelevance();
		
	}
	
	
	interface Result extends MetricAssociate {
		
		double getFitValue();
		
		double getRelevance();
		
		boolean isValid();
		
	}
	

}