package de.uni_potsdam.hpi.isg.efes.csg.datafit.structure;

import java.io.IOException;
import java.io.InputStream;
import java.sql.SQLException;

import org.apache.logging.log4j.LogManager;
import org.junit.Test;
import org.xml.sax.SAXException;

import de.uni_potsdam.hpi.isg.efes.csg.datafit.DataFitReport;
import de.uni_potsdam.hpi.isg.efes.csg.datafit.structure.StructureFitInspector;
import de.uni_potsdam.hpi.isg.efes.csg.datafit.structure.StructureFitInspector.Violation;
import de.uni_potsdam.hpi.isg.efes.csg.mapping.BasicNodeMapping;
import de.uni_potsdam.hpi.isg.efes.csg.mapping.RelationshipMatcher;
import de.uni_potsdam.hpi.isg.efes.csg.mapping.TableNodeMatcher;
import de.uni_potsdam.hpi.isg.efes.csg.scenario.DataSet;
import de.uni_potsdam.hpi.isg.efes.csg.scenario.IntegrationScenario;
import de.uni_potsdam.hpi.isg.efes.csg.scenario.adapter.SpicyConfigurationScenarioBuilder;
import de.uni_potsdam.hpi.isg.efes.spicy.SpicyConfiguration;
import de.uni_potsdam.hpi.isg.efes.spicy.SpicyParser;

public class StructureFitInspectorTest {

	@Test
	public void test() throws SQLException, SAXException, IOException {
		SpicyParser spicyParser = new SpicyParser();
		InputStream resourceAsStream = getClass().getResourceAsStream("/musicbrainz2discogs.xml");
		SpicyConfiguration spicyConfiguration = spicyParser.parse(resourceAsStream);

		SpicyConfigurationScenarioBuilder builder = new SpicyConfigurationScenarioBuilder();
		builder.add(spicyConfiguration);
		IntegrationScenario integrationScenario = builder.build();

		TableNodeMatcher enricher = new TableNodeMatcher();
		DataSet sourceDataSet = integrationScenario.getSourceDataSets().get(0);
		BasicNodeMapping mapping = integrationScenario.getMappings().get(sourceDataSet);
		DataSet targetDataSet = integrationScenario.getTargetDataSet();
		enricher.enrich(sourceDataSet.getSchema(), mapping, targetDataSet.getSchema());

		RelationshipMatcher relationshipMatcher = new RelationshipMatcher(integrationScenario, 0);
		relationshipMatcher.run();

		DataFitReport report = new DataFitReport();
		StructureFitInspector inspector = new StructureFitInspector();
		inspector.setIntegrationScenario(integrationScenario);
		inspector.evaluateScenarioTo(report);

		StructureFitInspector.ReportSection section = (StructureFitInspector.ReportSection) report
				.getSections().get(StructureFitInspector.ReportSection.class);
		for (Violation violation : section.getStructureViolations()) {
			LogManager.getRootLogger().debug(violation);
		}
	}

}
