package de.uni_potsdam.hpi.isg.efes.csg.datafit.values;

import java.io.IOException;
import java.io.InputStream;
import java.sql.SQLException;
import java.util.Set;

import org.junit.Test;
import org.xml.sax.SAXException;

import de.uni_potsdam.hpi.isg.efes.csg.datafit.DataFitReport;
import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.conclusion.SequentialConclusionModel;
import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.metrics.AverageStringLengthMetric;
import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.metrics.CharacterHistogramMetric;
import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.metrics.ConstancyMetric;
import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.metrics.ExponentialHistogramMetric;
import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.metrics.FillStatusMetric;
import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.metrics.MeanValueMetric;
import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.metrics.RangeOverlapMetric;
import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.metrics.TextPatternMetric;
import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.metrics.TopKValuesMetric;
import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.metrics.ValueFitMetric;
import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.metrics.ValueFitMetric.Result;
import de.uni_potsdam.hpi.isg.efes.csg.mapping.BasicNodeMapping;
import de.uni_potsdam.hpi.isg.efes.csg.mapping.NodeCorrespondence;
import de.uni_potsdam.hpi.isg.efes.csg.mapping.RelationshipMatcher;
import de.uni_potsdam.hpi.isg.efes.csg.mapping.TableNodeMatcher;
import de.uni_potsdam.hpi.isg.efes.csg.nodes.Node;
import de.uni_potsdam.hpi.isg.efes.csg.nodes.ValueNode;
import de.uni_potsdam.hpi.isg.efes.csg.scenario.DataSet;
import de.uni_potsdam.hpi.isg.efes.csg.scenario.IntegrationScenario;
import de.uni_potsdam.hpi.isg.efes.csg.scenario.adapter.SpicyConfigurationScenarioBuilder;
import de.uni_potsdam.hpi.isg.efes.spicy.SpicyConfiguration;
import de.uni_potsdam.hpi.isg.efes.spicy.SpicyParser;

public class ValueCharacteristicsCalibration {

	private static String[] testDataSets = { "/musicbrainz2discogs.xml", "/discogs2musicbrainz.xml", "/freedb2musicbrainz.xml",
			"/musicbrainz2freedb.xml", "/freedb2discogs.xml", "/freedb2freedb.xml", "/musicbrainz2musicbrainz.xml", "/discogs2discogs.xml" };

	@Test
	public void testCalibration() throws SQLException, SAXException, IOException {
		// outputConstancyValues(testDataSets[0]);
		outputStatisticValues(testDataSets[5]);
	}

	public static void outputConstancyValues(String dataSetPath) throws SQLException, SAXException, IOException {
		SpicyParser spicyParser = new SpicyParser();
		InputStream resourceAsStream = ValueCharacteristicsCalibration.class.getResourceAsStream(dataSetPath);
		SpicyConfiguration spicyConfiguration = spicyParser.parse(resourceAsStream);

		SpicyConfigurationScenarioBuilder builder = new SpicyConfigurationScenarioBuilder();
		builder.add(spicyConfiguration);
		IntegrationScenario integrationScenario = builder.build();
		DataSet sourceDataSet = integrationScenario.getSourceDataSets().get(0);
		DataSet targetDataSet = integrationScenario.getTargetDataSet();
		BasicNodeMapping mapping = integrationScenario.getMappings().get(sourceDataSet);

		// Create identity mapping (for discogs2discogs and
		// musicbrainz2musicbrainz)
		if (mapping.getNodeCorrespondences().isEmpty()) {
			mapping = BasicNodeMapping.createIdentityMapping(sourceDataSet, targetDataSet);
			integrationScenario.setMapping(sourceDataSet, mapping);
		}

		TableNodeMatcher enricher = new TableNodeMatcher();
		enricher.enrich(sourceDataSet.getSchema(), mapping, targetDataSet.getSchema());

		RelationshipMatcher relationshipMatcher = new RelationshipMatcher(integrationScenario, 0);
		relationshipMatcher.run();

		DataFitReport report = new DataFitReport();
		ValueCharacteristicsInspector inspector = new ValueCharacteristicsInspector();
		inspector.setIntegrationScenario(integrationScenario);
		inspector.register(new FillStatusMetric());
		inspector.register(new AverageStringLengthMetric());
		inspector.register(new CharacterHistogramMetric());
		inspector.register(new ConstancyMetric());
		inspector.register(new RangeOverlapMetric());
		inspector.register(new TextPatternMetric());
		inspector.register(new TopKValuesMetric(10));
		inspector.setConclusionModel(new SequentialConclusionModel());
		inspector.evaluateScenarioTo(report);

		ValueCharacteristicsInspector.ReportSection reportSection = (ValueCharacteristicsInspector.ReportSection) report
				.getSections().get(ValueCharacteristicsInspector.ReportSection.class);

		for (Node node : sourceDataSet.getSchema().getAtomicNodes().values()) {
			if (!node.isSimpleValueNode())
				continue;
			ConstancyMetric.ColumnStatistic constancy = (ConstancyMetric.ColumnStatistic) reportSection
					.getSourceColumnStatistic((ValueNode) node, ConstancyMetric.class);
			if (constancy == null)
				continue;
			FillStatusMetric.ColumnStatistic fillStatus = (FillStatusMetric.ColumnStatistic) reportSection
					.getSourceColumnStatistic((ValueNode) node, FillStatusMetric.class);
			System.out.format("%s;%s;%s\n", node.getName(), constancy.getConstancy(),
					fillStatus.getNumConvertibleValues());
		}
		System.out.println();
		for (Node node : targetDataSet.getSchema().getAtomicNodes().values()) {
			if (!node.isSimpleValueNode())
				continue;
			ConstancyMetric.ColumnStatistic constancy = (ConstancyMetric.ColumnStatistic) reportSection
					.getTargetColumnStatistic((ValueNode) node, ConstancyMetric.class);
			if (constancy == null)
				continue;
			FillStatusMetric.ColumnStatistic fillStatus = (FillStatusMetric.ColumnStatistic) reportSection
					.getTargetColumnStatistic((ValueNode) node, FillStatusMetric.class);
			System.out.format("%s;%s;%s\n", node.getName(), constancy.getConstancy(), fillStatus.getColumnSize()
					- fillStatus.getNumNullValues());
		}
	}

	@SuppressWarnings("unchecked")
	public static void outputStatisticValues(String dataSetPath) throws SQLException, SAXException, IOException {
		SpicyParser spicyParser = new SpicyParser();
		InputStream resourceAsStream = ValueCharacteristicsCalibration.class.getResourceAsStream(dataSetPath);
		SpicyConfiguration spicyConfiguration = spicyParser.parse(resourceAsStream);

		SpicyConfigurationScenarioBuilder builder = new SpicyConfigurationScenarioBuilder();
		builder.add(spicyConfiguration);
		IntegrationScenario integrationScenario = builder.build();
		DataSet sourceDataSet = integrationScenario.getSourceDataSets().get(0);
		DataSet targetDataSet = integrationScenario.getTargetDataSet();
		BasicNodeMapping mapping = integrationScenario.getMappings().get(sourceDataSet);

		// Create identity mapping (for discogs2discogs and
		// musicbrainz2musicbrainz)
		if (mapping.getNodeCorrespondences().isEmpty()) {
			mapping = BasicNodeMapping.createIdentityMapping(sourceDataSet, targetDataSet);
			integrationScenario.setMapping(sourceDataSet, mapping);
		}

		TableNodeMatcher enricher = new TableNodeMatcher();
		enricher.enrich(sourceDataSet.getSchema(), mapping, targetDataSet.getSchema());

		RelationshipMatcher relationshipMatcher = new RelationshipMatcher(integrationScenario, 0);
		relationshipMatcher.run();

		DataFitReport report = new DataFitReport();
		ValueCharacteristicsInspector inspector = new ValueCharacteristicsInspector();
		inspector.setIntegrationScenario(integrationScenario);
		inspector.register(new FillStatusMetric());
		inspector.register(new AverageStringLengthMetric());
		inspector.register(new CharacterHistogramMetric());
		inspector.register(new ConstancyMetric());
		inspector.register(new RangeOverlapMetric());
		inspector.register(new TextPatternMetric());
		inspector.register(new TopKValuesMetric(20));
		inspector.register(new MeanValueMetric());
		inspector.register(new ExponentialHistogramMetric());
		inspector.setConclusionModel(new SequentialConclusionModel());
		inspector.evaluateScenarioTo(report);

		ValueCharacteristicsInspector.ReportSection reportSection = (ValueCharacteristicsInspector.ReportSection) report
				.getSections().get(ValueCharacteristicsInspector.ReportSection.class);
		Set<NodeCorrespondence> nodeCorrespondences = reportSection.getMetricResults().keySet();
		final Class<?>[] metricClasses = { AverageStringLengthMetric.class, CharacterHistogramMetric.class,
				TextPatternMetric.class, TopKValuesMetric.class, RangeOverlapMetric.class, MeanValueMetric.class,
				ExponentialHistogramMetric.class };
		System.out.print("source;target;sourcevals;targetvals");
		for (Class<?> metricClass : metricClasses) {
			System.out.print(";" + metricClass.getSimpleName() + ";rel-" + metricClass.getSimpleName());
		}
		System.out.println();
		for (NodeCorrespondence nodeCorrespondence : nodeCorrespondences) {
			Node sourceNode = nodeCorrespondence.getSourceNode();
			Node targetNode = nodeCorrespondence.getTargetNode();
			System.out.print(sourceNode + ";" + targetNode);
			FillStatusMetric.ColumnStatistic fillStatus = (FillStatusMetric.ColumnStatistic) reportSection
					.getSourceColumnStatistic((ValueNode) sourceNode, FillStatusMetric.class);
			System.out.print(";" + (fillStatus.getColumnSize() - fillStatus.getNumNullValues()));
			fillStatus = (FillStatusMetric.ColumnStatistic) reportSection.getTargetColumnStatistic(
					(ValueNode) targetNode, FillStatusMetric.class);
			System.out.print(";" + (fillStatus.getColumnSize() - fillStatus.getNumNullValues()));
			for (Class<?> metricClass : metricClasses) {
				System.out.print(";");
				Result result = reportSection.getResult(nodeCorrespondence,
						(Class<? extends ValueFitMetric>) metricClass);
				if (result != null && result.isValid()) {
					System.out.print(result.getFitValue());
					System.out.print(";");
					System.out.print(result.getRelevance());
				} else {
					System.out.print(";");
				}
			}
			System.out.println();
		}
	}

}
