package de.uni_potsdam.hpi.isg.efes.csg.datafit.values;

import java.io.IOException;
import java.io.InputStream;
import java.sql.SQLException;

import org.junit.Test;
import org.xml.sax.SAXException;

import de.uni_potsdam.hpi.isg.efes.csg.datafit.DataFitReport;
import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.conclusion.SequentialConclusionModel;
import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.metrics.AverageStringLengthMetric;
import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.metrics.CharacterHistogramMetric;
import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.metrics.ConstancyMetric;
import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.metrics.FillStatusMetric;
import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.metrics.RangeOverlapMetric;
import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.metrics.TextPatternMetric;
import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.metrics.TopKValuesMetric;
import de.uni_potsdam.hpi.isg.efes.csg.mapping.BasicNodeMapping;
import de.uni_potsdam.hpi.isg.efes.csg.mapping.RelationshipMatcher;
import de.uni_potsdam.hpi.isg.efes.csg.mapping.TableNodeMatcher;
import de.uni_potsdam.hpi.isg.efes.csg.scenario.DataSet;
import de.uni_potsdam.hpi.isg.efes.csg.scenario.IntegrationScenario;
import de.uni_potsdam.hpi.isg.efes.csg.scenario.adapter.SpicyConfigurationScenarioBuilder;
import de.uni_potsdam.hpi.isg.efes.spicy.SpicyConfiguration;
import de.uni_potsdam.hpi.isg.efes.spicy.SpicyParser;

public class ValueCharacteristicsInspectorTest {

	@Test
	public void test() throws SQLException, SAXException, IOException {
		SpicyParser spicyParser = new SpicyParser();
//		InputStream resourceAsStream = getClass().getResourceAsStream("/musicbrainz2discogs.xml");
		InputStream resourceAsStream = getClass().getResourceAsStream("/freedb2musicbrainz.xml");
//		InputStream resourceAsStream = getClass().getResourceAsStream("/musicbrainz2freedb.xml");
		SpicyConfiguration spicyConfiguration = spicyParser.parse(resourceAsStream);

		SpicyConfigurationScenarioBuilder builder = new SpicyConfigurationScenarioBuilder();
		builder.add(spicyConfiguration);
		IntegrationScenario integrationScenario = builder.build();

		TableNodeMatcher enricher = new TableNodeMatcher();
		DataSet sourceDataSet = integrationScenario.getSourceDataSets().get(0);
		BasicNodeMapping mapping = integrationScenario.getMappings().get(sourceDataSet);
		DataSet targetDataSet = integrationScenario.getTargetDataSet();
		enricher.enrich(sourceDataSet.getSchema(), mapping, targetDataSet.getSchema());

		RelationshipMatcher relationshipMatcher = new RelationshipMatcher(integrationScenario, 0);
		relationshipMatcher.run();

		DataFitReport report = new DataFitReport();
		ValueCharacteristicsInspector inspector = new ValueCharacteristicsInspector();
		inspector.setIntegrationScenario(integrationScenario);
		inspector.register(new FillStatusMetric());
		inspector.register(new AverageStringLengthMetric());
		inspector.register(new CharacterHistogramMetric());
		inspector.register(new ConstancyMetric());
		inspector.register(new RangeOverlapMetric());
		inspector.register(new TextPatternMetric());
		inspector.register(new TopKValuesMetric(10));
		inspector.setConclusionModel(new SequentialConclusionModel());
		inspector.evaluateScenarioTo(report);
	}

}
