package de.uni_potsdam.hpi.isg.efes.csg.effort.format;

import java.util.ArrayList;
import java.util.Collection;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.uni_potsdam.hpi.isg.efes.csg.effort.tasklist.TaskList.Task;

public class Formatters {
	
	private static final Logger logger = LogManager.getLogger(Formatters.class.getSimpleName());
	
	private static final Collection<EffortFormatter> formatters = new ArrayList<>();
	static {
		formatters.add(new TimeFormatter());
	}
	
	public static EffortFormatter getFormatter(String unit) {
		for (EffortFormatter formatter : formatters) {
			if (formatter.supports(unit)) {
				return formatter;
			}
		}
		logger.warn("Could not find a formatter for unit '{}', using fallback formatter.", unit);
		return FALLBACK_FORMATTER;
	}
	
	private static final EffortFormatter FALLBACK_FORMATTER = new EffortFormatter() {
		
		@Override
		public boolean supports(String unit) {
			return true;
		}
		
		@Override
		public String formatEffort(double value, String unit) {
			if (value < 0) {
				return "n/a";
			}
			return String.format("%.2f %s", value, unit);
		}
		
		@Override
		public String formatEffort(Task task, String unit) {
			return formatEffort(task.getEstimationValue(), unit);
		}
	};
	

}
