package de.uni_potsdam.hpi.isg.efes.csg.effort.format;

import java.util.HashSet;
import java.util.Set;

import de.uni_potsdam.hpi.isg.efes.csg.effort.tasklist.TaskList.Task;

public class TimeFormatter implements EffortFormatter {

	private static final Set<String> supportedFormats = new HashSet<>();
	static {
		supportedFormats.add("h");
		supportedFormats.add("min");
	}
	
	@Override
	public boolean supports(String unit) {
		return supportedFormats.contains(unit);
	}

	@Override
	public String formatEffort(double value, String unit) {
		if (value == Task.NO_ESTIMATE) {
			return "n/a";
		}
		
		int minutes = convertToMinutes(value, unit);
		int hours = minutes / 60;
		minutes %= 60;
		
		return String.format("%2dh %02dmin", hours, minutes);
	}
	
	private int convertToMinutes(double value, String unit) {
		switch (unit) {
		case "h":
			return (int) Math.round(value * 60);
		case "min":
			return (int) Math.round(value);
		default:
			throw new IllegalArgumentException("Illegal time unit: " + unit);
		}
	}

	@Override
	public String formatEffort(Task task, String unit) {
		return formatEffort(task.getEstimationValue(), unit);
	}

}
