package de.uni_potsdam.hpi.isg.efes.csg.effort.tasklist;

import java.util.Collection;
import java.util.Collections;
import java.util.Set;

import de.uni_potsdam.hpi.isg.efes.csg.datafit.DataFitReport.Section;
import de.uni_potsdam.hpi.isg.efes.csg.datafit.mapping.MappingInspector;
import de.uni_potsdam.hpi.isg.efes.csg.effort.tasklist.TaskListModel.Estimate;
import de.uni_potsdam.hpi.isg.efes.csg.effort.tasklist.TaskListModel.Profile;
import de.uni_potsdam.hpi.isg.efes.csg.effort.tasklist.TaskListModel.Submodel;
import de.uni_potsdam.hpi.isg.efes.csg.relationships.Relationship;
import de.uni_potsdam.hpi.isg.efes.csg.scenario.IntegrationScenario;

public class SimpleMappingModel implements Submodel {

	public static final String CATEGORY = "Mapping Creation";
	
	private static final Set<Class<? extends Section>> SUPPORTED_REPORT_CLASSES = Collections.<Class<? extends Section>>singleton(MappingInspector.ReportSection.class);
	@SuppressWarnings("unused")
	private IntegrationScenario integrationScenario;

	@Override
	public Collection<Class<? extends Section>> getSupportedReportSectionClasses() {
		return SUPPORTED_REPORT_CLASSES;
	}

	@Override
	public void setIntegrationScenario(IntegrationScenario integrationScenario) {
		this.integrationScenario = integrationScenario;
	}

	@Override
	public void setProfile(Profile profile) {
		// Profile is ignored for this task.
	}

	@Override
	public TaskList createTaskList(Section section, Estimate estimate) {
		TaskList taskList = new TaskList();

		MappingInspector.ReportSection mappingSection = (MappingInspector.ReportSection) section;
		for (MappingInspector.TableMapping tableMapping : mappingSection.getTableMappings()) {
			TaskList.Task task = new TaskList.Task("Write mapping", SimpleMappingModel.CATEGORY, tableMapping.getTableNode().getName());
			task.setParameter("pk", tableMapping.hasMetaDataPrimaryKey() ? 1 : 0);
			task.setParameter("fks", tableMapping.getNumMetaDataForeignKeyAttributes());
			task.setParameter("meta", tableMapping.getNumOtherMetaData());
			task.setParameter("attributes", tableMapping.getMappedRelationships().size());
			task.setParameter("tables", tableMapping.getNumInvolvedSourceTables());
			int complexitySum = 0;
			for (Relationship mappedRelationship : tableMapping.getMappedRelationships()) {
				complexitySum += mappedRelationship.getComplexity();
			}
			task.setParameter("complexity", complexitySum);
			taskList.add(task);
		}
		
		return taskList;
	}

}
