package de.uni_potsdam.hpi.isg.efes.csg.effort.tasklist;

import it.unimi.dsi.fastutil.objects.Object2IntMap;
import it.unimi.dsi.fastutil.objects.Object2IntOpenHashMap;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import de.uni_potsdam.hpi.isg.utils.util.Strings;

public class TaskList {

	private List<Task> tasks = new ArrayList<>();

	public void add(Task task) {
		tasks.add(task);
	}

	public List<Task> getTasks() {
		return Collections.unmodifiableList(this.tasks);
	}

	@Override
	public String toString() {
		return "TaskList" + tasks;
	}

	public static class Task {

		public static final int NO_ESTIMATE = -1;

		private String name;
		
		private String category;

		private String notes;

		private double estimationValue = NO_ESTIMATE;

		private Object2IntMap<String> parameters;
		
		public Task(String name, String category, String... notes) {
			this(name, category, new Object2IntOpenHashMap<String>(), notes);
		}
		
		public Task(String name, String category, int repetitions, String... notes) {
			this(name, category, new Object2IntOpenHashMap<String>(), notes);
			setRepetitions(repetitions);
		}

		public Task(String name, String category, Object2IntMap<String> parameters, String... notes) {
			this.name = name;
			this.category = category;
			this.parameters = parameters;
			this.notes = Strings.join(Arrays.asList(notes), ", ");
		}
		
		public void setRepetitions(int repetitions) {
			setParameter("repetitions", repetitions);
		}
		
		public int getRepetitions() {
			return this.parameters.getInt("repetitions");
		}
		
		public void setParameter(String key, int value) {
			this.parameters.put(key, value);
		}

		public String getName() {
			return name;
		}
		
		public String getCategory() {
			return category;
		}

		public Object2IntMap<String> getParameters() {
			return parameters;
		}

		public double getEstimationValue() {
			return estimationValue;
		}

		public boolean hasEstimationValue() {
			return this.estimationValue != NO_ESTIMATE;
		}

		public void setEstimationValue(double estimationValue) {
			this.estimationValue = estimationValue;
		}

		public String getNotes() {
			return notes;
		}

		@Override
		public String toString() {
			return "Task [name=" + name + ", notes=" + notes + ", parameters=" + parameters + "]";
		}

	}

}
