package de.uni_potsdam.hpi.isg.efes.csg.effort.tasklist.repair;

import java.util.Arrays;
import java.util.Collection;
import java.util.List;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.uni_potsdam.hpi.isg.efes.csg.constraints.Constraint;
import de.uni_potsdam.hpi.isg.efes.csg.constraints.NaturalNumberSet;
import de.uni_potsdam.hpi.isg.efes.csg.effort.tasklist.TaskListModel.Profile;
import de.uni_potsdam.hpi.isg.efes.csg.instance.SchemaGraphInstance;
import de.uni_potsdam.hpi.isg.efes.csg.nodes.IdentityNode;
import de.uni_potsdam.hpi.isg.efes.csg.relationships.ExplicitAtomicRelationship;
import de.uni_potsdam.hpi.isg.efes.csg.relationships.Relationship;

public class AddMissingTuples extends AbstractSchemaGraphOperationDescriptor<AddMissingTuples.Operation> {

	@SuppressWarnings("unused")
	private static final Logger logger = LogManager.getLogger(AddMissingTuples.class.getSimpleName());

	private static final NaturalNumberSet TARGET_CARDINALITY = NaturalNumberSet.GREATER_OR_EQUAL_ONE;
	private static final List<Profile> PROFILES = Arrays.asList(Profile.HIGH_QUALITY);
	private static final Class<?>[] GRAPH_PATTERN = new Class<?>[] { IdentityNode.class,
			ExplicitAtomicRelationship.class, IdentityNode.class };

	@Override
	protected Class<?>[] getGraphPattern() {
		return GRAPH_PATTERN;
	}

	@Override
	List<Profile> getProfiles() {
		return PROFILES;
	}

	@Override
	protected Operation createOperation(Constraint constraint, RepairSchemaGraphInstance sgInstance) {
		return new Operation(constraint.getRelationship(), sgInstance);
	}

	@Override
	protected NaturalNumberSet getTargetCardinality() {
		return TARGET_CARDINALITY;
	}

	public class Operation extends AbstractSchemaGraphOperationDescriptor<Operation>.AbstractSchemaGraphOperation {

		public Operation(Relationship targetRelationship, RepairSchemaGraphInstance instance) {
			super(targetRelationship, instance);
		}

		@Override
		public void searchSideEffectRelationships(SchemaGraphInstance sgi) {
			Relationship targetInverse = getTargetRelationship().getInverse();
			addSideEffectRelationship(targetInverse, INVERSE);

			Collection<Relationship> relationships = sgi.getSchema().getAtomicRelationships(
					getTargetRelationship().getEndNode());

			for (Relationship relationship : relationships) {
				if (!relationship.isAtomic() || relationship.equals(targetInverse)) {
					continue;
				}

				if (relationship.isExplicit() && relationship.getEndNode().isSimpleIdentityNode()) {
					addSideEffectRelationship(relationship, END_TABLE_TO_TABLE);
				} else if (relationship.isExplicit() && relationship.getEndNode().isSimpleValueNode()) {
					addSideEffectRelationship(relationship, END_TABLE_TO_ATTRIBUTE);
				}
			}
		}

		@Override
		protected NaturalNumberSet calculateNewTargetCardinality(NaturalNumberSet oldCardinality) {
			return oldCardinality.intersect(NaturalNumberSet.GREATER_OR_EQUAL_ONE);
		}

		@Override
		protected NaturalNumberSet calculateNewSideEffectCardinality(NaturalNumberSet oldCardinality, int type) {
			switch (type) {
			case INVERSE:
				return oldCardinality.union(NaturalNumberSet.ONE);

			case END_TABLE_TO_TABLE:
			case END_TABLE_TO_ATTRIBUTE:
				return oldCardinality.add(NaturalNumberSet.ZERO);

			default:
				throw new IllegalArgumentException("Unsupported side effect type: " + type);
			}
		}

		@Override
		protected void executeOnBags() {
			// Process fix.
			Relationship targetRelationship = getTargetRelationship();
			int numAddedElements = addEndElementsForUnlinkedStartElements(targetRelationship);
			addSingleLinkedElements(targetRelationship.getInverse(), numAddedElements);

			// Handle side effects on attributes of newly created tuples.
			Collection<Relationship> tableToAttributeRelationships = this
					.getSideEffectRelationships(END_TABLE_TO_ATTRIBUTE);
			for (Relationship tableToAttributeRelationship : tableToAttributeRelationships) {
				handleElementAdditionSideEffects(tableToAttributeRelationship, numAddedElements);
			}
			
			
			// Handle side effects on table linked to the affected table.
			Collection<Relationship> tableToTableRelationships = this
					.getSideEffectRelationships(END_TABLE_TO_TABLE);
			for (Relationship tableToTableRelationship : tableToTableRelationships) {
				handleElementAdditionSideEffects(tableToTableRelationship, numAddedElements);
			}
		}

	}

}
