package de.uni_potsdam.hpi.isg.efes.csg.effort.tasklist.repair;

import java.util.Arrays;
import java.util.Collection;
import java.util.List;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.uni_potsdam.hpi.isg.efes.csg.constraints.Constraint;
import de.uni_potsdam.hpi.isg.efes.csg.constraints.NaturalNumberSet;
import de.uni_potsdam.hpi.isg.efes.csg.effort.tasklist.TaskListModel.Profile;
import de.uni_potsdam.hpi.isg.efes.csg.instance.SchemaGraphInstance;
import de.uni_potsdam.hpi.isg.efes.csg.nodes.IdentityNode;
import de.uni_potsdam.hpi.isg.efes.csg.nodes.Node;
import de.uni_potsdam.hpi.isg.efes.csg.nodes.ValueNode;
import de.uni_potsdam.hpi.isg.efes.csg.relationships.Relationship;
import de.uni_potsdam.hpi.isg.efes.csg.relationships.ExplicitAtomicRelationship;

public class CreateTuplesForDetachedValues extends
		AbstractSchemaGraphOperationDescriptor<CreateTuplesForDetachedValues.Operation> {

	@SuppressWarnings("unused")
	private static final Logger logger = LogManager.getLogger(CreateTuplesForDetachedValues.class.getSimpleName());

	private static final Class<?>[] GRAPH_PATTERN = new Class<?>[] { ValueNode.class, ExplicitAtomicRelationship.class,
			IdentityNode.class };
	private static final List<Profile> PROFILES = Arrays.asList(Profile.HIGH_QUALITY);
	private static final NaturalNumberSet TARGET_CARDINALITY = NaturalNumberSet.GREATER_OR_EQUAL_ONE;

	@Override
	protected Class<?>[] getGraphPattern() {
		return GRAPH_PATTERN;
	}

	@Override
	List<Profile> getProfiles() {
		return PROFILES;
	}

	@Override
	protected Operation createOperation(
			Constraint constraint, RepairSchemaGraphInstance sgInstance) {
		return new Operation(constraint.getRelationship(), sgInstance);
	}

	@Override
	protected NaturalNumberSet getTargetCardinality() {
		return TARGET_CARDINALITY;
	}

	public class Operation extends AbstractSchemaGraphOperationDescriptor<Operation>.AbstractSchemaGraphOperation {

		public Operation(Relationship targetRelationship, RepairSchemaGraphInstance instance) {
			super(targetRelationship, instance);
		}

		@Override
		public void searchSideEffectRelationships(SchemaGraphInstance sgi) {
			addSideEffectRelationship(getTargetRelationship().getInverse(), INVERSE);

			Node tableNode = getTargetRelationship().getEndNode();
			for (Relationship relationship : sgi.getSchema().getAtomicRelationships(tableNode)) {
				if (!relationship.isExplicit()) {
					continue;
				}

				if (!relationship.equals(getTargetRelationship().getInverse()) && relationship.isExplicit()
						&& relationship.getEndNode().isSimpleValueNode()) {
					addSideEffectRelationship(relationship, END_TABLE_TO_ATTRIBUTE);
				} else if (relationship.isExplicit() && relationship.getEndNode().isSimpleIdentityNode()) {
					addSideEffectRelationship(relationship, END_TABLE_TO_TABLE);
				}
			}
		}

		@Override
		protected NaturalNumberSet calculateNewTargetCardinality(NaturalNumberSet oldCardinality) {
			return oldCardinality.intersect(NaturalNumberSet.GREATER_OR_EQUAL_ONE).union(NaturalNumberSet.ONE);
		}

		@Override
		protected NaturalNumberSet calculateNewSideEffectCardinality(NaturalNumberSet oldCardinality, int type) {
			switch (type) {
			case INVERSE:
				return oldCardinality.union(NaturalNumberSet.ONE);
			case END_TABLE_TO_ATTRIBUTE:
			case END_TABLE_TO_TABLE:
				return oldCardinality.union(NaturalNumberSet.ZERO);
			default:
				throw new IllegalArgumentException("Unsupported side effect type: " + type);
			}
		}

		@Override
		protected void executeOnBags() {
			// Process fix.
			Relationship targetRelationship = getTargetRelationship();
			int numAddedElements = addEndElementsForUnlinkedStartElements(targetRelationship);
			addSingleLinkedElements(targetRelationship.getInverse(), numAddedElements);

			// Handle side effects on attributes of newly created tuples.
			Collection<Relationship> tableToAttributeRelationships = this
					.getSideEffectRelationships(END_TABLE_TO_ATTRIBUTE);
			for (Relationship tableToAttributeRelationship : tableToAttributeRelationships) {
				handleElementAdditionSideEffects(tableToAttributeRelationship, numAddedElements);
			}
			
			
			// Handle side effects on table linked to the affected table.
			Collection<Relationship> tableToTableRelationships = this
					.getSideEffectRelationships(END_TABLE_TO_TABLE);
			for (Relationship tableToTableRelationship : tableToTableRelationships) {
				handleElementAdditionSideEffects(tableToTableRelationship, numAddedElements);
			}
			
		}

	}

}
