package de.uni_potsdam.hpi.isg.efes.csg.effort.tasklist.repair;

import java.util.Arrays;
import java.util.List;

import de.uni_potsdam.hpi.isg.efes.csg.constraints.Constraint;
import de.uni_potsdam.hpi.isg.efes.csg.constraints.NaturalNumberSet;
import de.uni_potsdam.hpi.isg.efes.csg.effort.tasklist.TaskListModel.Profile;
import de.uni_potsdam.hpi.isg.efes.csg.instance.SchemaGraphInstance;
import de.uni_potsdam.hpi.isg.efes.csg.nodes.Node;
import de.uni_potsdam.hpi.isg.efes.csg.relationships.Relationship;

public class FallbackFix extends AbstractSchemaGraphOperationDescriptor<FallbackFix.Operation> {

	private NaturalNumberSet targetCardinality;
	
	public FallbackFix(NaturalNumberSet targetCardinality) {
		super();
		this.targetCardinality = targetCardinality;
	}

	@Override
	protected Class<?>[] getGraphPattern() {
		return new Class<?>[] { Node.class, Relationship.class, Node.class };
	}

	@Override
	List<Profile> getProfiles() {
		return Arrays.asList(Profile.HIGH_QUALITY, Profile.LOW_EFFORT);
	}

	@Override
	protected Operation createOperation(Constraint constraint, RepairSchemaGraphInstance sgInstance) {
		return new Operation(constraint, sgInstance);
	}

	@Override
	protected NaturalNumberSet getTargetCardinality() {
		return this.targetCardinality;
	}

	public class Operation extends AbstractSchemaGraphOperationDescriptor<Operation>.AbstractSchemaGraphOperation {
	
		public Operation(Constraint constraint, RepairSchemaGraphInstance instance) {
			super(constraint.getRelationship(), instance);
		}
	
		@Override
		public void searchSideEffectRelationships(SchemaGraphInstance sgi) {
			// No side effects.
		}
	
		@Override
		protected NaturalNumberSet calculateNewTargetCardinality(NaturalNumberSet oldCardinality) {
			return oldCardinality.intersect(targetCardinality);
		}
	
		@Override
		protected NaturalNumberSet calculateNewSideEffectCardinality(NaturalNumberSet oldCardinality, int type) {
			// No side effects.
			throw new UnsupportedOperationException("No side effects supported for fallback fixes.");
		}
	
	}

}
