package de.uni_potsdam.hpi.isg.efes.csg.effort.tasklist.simple;

import de.uni_potsdam.hpi.isg.efes.csg.datafit.simple.SimpleReportSection;
import de.uni_potsdam.hpi.isg.efes.csg.datafit.simple.SimpleReportSection.DataSet;
import de.uni_potsdam.hpi.isg.efes.csg.effort.tasklist.TaskList;
import de.uni_potsdam.hpi.isg.efes.csg.effort.tasklist.TaskList.Task;

abstract public class TaskType {
	
	private String name;
	
	private String category;

	protected TaskType(String name, String category) {
		this.name = name;
		this.category = category;
	}
	
	public static TaskType createTaskType(String name, String category, String scope) {
		if ("per-source".equals(scope)) {
			return new PerSourceTaskType(name, category);
		}
		
		throw new IllegalArgumentException("Unsupported task type scope: "+ scope);
	}
	
	protected Task createTask(String... notes) {
		return new Task(this.name, this.category, notes);
	}
	
	abstract public void addTasks(TaskList taskList, SimpleReportSection reportSection);
	
	private static class PerSourceTaskType extends TaskType {
		
		protected PerSourceTaskType(String name, String category) {
			super(name, category);
		}

		@Override
		public void addTasks(TaskList taskList, SimpleReportSection reportSection) {
			for (DataSet source : reportSection.getSourceDataSets()) {
				Task task = createTask(source.toString());
				task.setParameter("attributes", source.getNumAttributes());
				task.setParameter("metadataAttributes", source.getNumMetaDataAttributes());
				task.setParameter("correspondences", source.getNumCorrespondences());
				task.setParameter("tables", source.getNumCorrespondences());
				taskList.add(task);
			}
		}
		
	}

}
