package de.uni_potsdam.hpi.isg.efes.csg.effort.tasklist.transform;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Set;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.uni_potsdam.hpi.isg.efes.csg.datafit.DataFitReport.Section;
import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.ValueCharacteristicsInspector;
import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.ValueCharacteristicsInspector.Issue;
import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.ValueCharacteristicsInspector.ReportSection;
import de.uni_potsdam.hpi.isg.efes.csg.effort.tasklist.TaskList;
import de.uni_potsdam.hpi.isg.efes.csg.effort.tasklist.TaskListModel;
import de.uni_potsdam.hpi.isg.efes.csg.effort.tasklist.TaskListModel.Estimate;
import de.uni_potsdam.hpi.isg.efes.csg.scenario.IntegrationScenario;

public class TransformModel implements TaskListModel.Submodel {

	public static final String CATEGORY = "Value Transformations";
	
	private static final Set<Class<? extends Section>> SUPPORTED_SECTIONS = Collections
			.<Class<? extends Section>> singleton(ValueCharacteristicsInspector.ReportSection.class);

	private static Logger logger = LogManager.getLogger(TransformModel.class.getSimpleName());

	@SuppressWarnings("unused")
	private IntegrationScenario integrationScenario;
	private TaskListModel.Profile profile;
	private static final Collection<TransformationOperation> transformOperations = new ArrayList<>();
	
	static {
		register(new ConvertValues());
		register(new AddValues());
		register(new DropValues());
		register(new GeneralizeValues());
		register(new RefineValues());
	}
	
	@Override
	public void setIntegrationScenario(IntegrationScenario integrationScenario) {
		this.integrationScenario = integrationScenario;
	}

	public void setProfile(TaskListModel.Profile profile) {
		this.profile = profile;
	}

	/**
	 * From the registered repair tasks, this function chooses an appropriate
	 * one for each violation and applies it.
	 * 
	 * @param section
	 */
	public TaskList createTaskList(Section section, Estimate estimate) {
		ValueCharacteristicsInspector.ReportSection valueCharacteristicsSection = (ReportSection) section;
		TaskList taskList = new TaskList();
		
		for (Issue issue : valueCharacteristicsSection.getIssues()) {
			TransformationOperation operation = findTransformationOperation(issue);
			if (operation == null) {
				logger.info("No transformation operation found for {}. It will be ignored.", issue);
			} else {
				taskList.add(operation.toTask(issue));
			}
		}

		return taskList;
	}
	
	private TransformationOperation findTransformationOperation(Issue issue) {
		for (TransformationOperation operation : transformOperations) {
			if (operation.supportsProfile(this.profile) && operation.canHandle(issue)) {
				return operation;
			}
		}
		return null;
	}

	@Override
	public Collection<Class<? extends Section>> getSupportedReportSectionClasses() {
		return SUPPORTED_SECTIONS;
	}

	public static void register(TransformationOperation descriptor) {
		transformOperations.add(descriptor);
	}
	
}
