package de.uni_potsdam.hpi.isg.efes.csg.effort.tasklist;

import java.io.IOException;
import java.io.InputStream;
import java.sql.SQLException;

import org.junit.Test;
import org.xml.sax.SAXException;

import de.uni_potsdam.hpi.isg.efes.csg.datafit.DataFitReport;
import de.uni_potsdam.hpi.isg.efes.csg.datafit.structure.StructureFitInspector;
import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.ValueCharacteristicsInspector;
import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.conclusion.SequentialConclusionModel;
import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.metrics.AverageStringLengthMetric;
import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.metrics.CharacterHistogramMetric;
import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.metrics.ConstancyMetric;
import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.metrics.FillStatusMetric;
import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.metrics.RangeOverlapMetric;
import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.metrics.TextPatternMetric;
import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.metrics.TopKValuesMetric;
import de.uni_potsdam.hpi.isg.efes.csg.effort.tasklist.TaskListModel.Estimate;
import de.uni_potsdam.hpi.isg.efes.csg.effort.tasklist.TaskListModel.Profile;
import de.uni_potsdam.hpi.isg.efes.csg.effort.tasklist.repair.RepairModel;
import de.uni_potsdam.hpi.isg.efes.csg.effort.tasklist.transform.TransformModel;
import de.uni_potsdam.hpi.isg.efes.csg.mapping.BasicNodeMapping;
import de.uni_potsdam.hpi.isg.efes.csg.mapping.RelationshipMatcher;
import de.uni_potsdam.hpi.isg.efes.csg.mapping.TableNodeMatcher;
import de.uni_potsdam.hpi.isg.efes.csg.scenario.DataSet;
import de.uni_potsdam.hpi.isg.efes.csg.scenario.IntegrationScenario;
import de.uni_potsdam.hpi.isg.efes.csg.scenario.adapter.SpicyConfigurationScenarioBuilder;
import de.uni_potsdam.hpi.isg.efes.spicy.SpicyConfiguration;
import de.uni_potsdam.hpi.isg.efes.spicy.SpicyParser;

public class TaskListModelTest {

	@Test
	public void testModel2() throws SQLException, SAXException, IOException {
		SpicyParser spicyParser = new SpicyParser();
//		InputStream resourceAsStream = getClass().getResourceAsStream("/discogs2freedb.xml");
		InputStream resourceAsStream = getClass().getResourceAsStream("/freedb2freedb.xml");
//		InputStream resourceAsStream = getClass().getResourceAsStream("/musicbrainz2discogs.xml");
		SpicyConfiguration spicyConfiguration = spicyParser.parse(resourceAsStream);

		SpicyConfigurationScenarioBuilder builder = new SpicyConfigurationScenarioBuilder();
		builder.add(spicyConfiguration);
		IntegrationScenario integrationScenario = builder.build();

		TableNodeMatcher tableNodeMatcher = new TableNodeMatcher();
		DataSet sourceDataSet = integrationScenario.getSourceDataSets().get(0);
		BasicNodeMapping mapping = integrationScenario.getMappings().get(sourceDataSet);
		DataSet targetDataSet = integrationScenario.getTargetDataSet();
		tableNodeMatcher.enrich(sourceDataSet.getSchema(), mapping, targetDataSet.getSchema());

		RelationshipMatcher relationshipMatcher = new RelationshipMatcher(integrationScenario, 0);
		relationshipMatcher.run();

		DataFitReport report = new DataFitReport();
		StructureFitInspector structureFitInspector = new StructureFitInspector();
		structureFitInspector.setIntegrationScenario(integrationScenario);
		structureFitInspector.evaluateScenarioTo(report);

		ValueCharacteristicsInspector valueCharacteristicsInspector = new ValueCharacteristicsInspector();
		valueCharacteristicsInspector.setIntegrationScenario(integrationScenario);
		valueCharacteristicsInspector.setIntegrationScenario(integrationScenario);
		valueCharacteristicsInspector.register(new FillStatusMetric());
		valueCharacteristicsInspector.register(new AverageStringLengthMetric());
		valueCharacteristicsInspector.register(new CharacterHistogramMetric());
		valueCharacteristicsInspector.register(new ConstancyMetric());
		valueCharacteristicsInspector.register(new RangeOverlapMetric());
		valueCharacteristicsInspector.register(new TextPatternMetric());
		valueCharacteristicsInspector.register(new TopKValuesMetric(10));
		valueCharacteristicsInspector.setConclusionModel(new SequentialConclusionModel());
		valueCharacteristicsInspector.evaluateScenarioTo(report);

		Profile profile;
		profile = TaskListModel.Profile.HIGH_QUALITY;
//		profile = TaskListModel.Profile.LOW_EFFORT;
		TaskListModel taskListModel = new TaskListModel(profile);
		taskListModel.setIntegrationScenario(integrationScenario);

		taskListModel.register(TaskEstimationFunction.createJuelExpressionFunction("AggregateValues", "h",
				"${0.1*repetitions}"));
		taskListModel.register(TaskEstimationFunction.createJuelExpressionFunction("Convert values", "h",
				"${0.01*values*complexity}"));
		taskListModel.register(TaskEstimationFunction.createJuelExpressionFunction("Generalize values", "h",
				"${0.01*distinctvalues}"));
		taskListModel.register(TaskEstimationFunction.createJuelExpressionFunction("Refine values", "h",
				"${0.01*values*complexity}"));
		taskListModel.register(TaskEstimationFunction.createJuelExpressionFunction("Drop values", "h",
				"${0.01*values}"));
		taskListModel
				.register(TaskEstimationFunction.createJuelExpressionFunction("Add values", "h", "${0.03*values}"));
		taskListModel.register(TaskEstimationFunction.createJuelExpressionFunction("AddMissingUniqueValues", "h",
				"${0.1*repetitions}"));
		taskListModel.register(TaskEstimationFunction.createJuelExpressionFunction("CreateTuplesForDetachedValues",
				"h", "${0.2}"));
		taskListModel.register(TaskEstimationFunction.createJuelExpressionFunction("KeepAnyValue",
				"h", "${0.1}"));
		taskListModel.register(TaskEstimationFunction.createJuelExpressionFunction("DeleteTuplesWithMissingValue",
				"h", "${0.1}"));
		taskListModel.register(TaskEstimationFunction.createJuelExpressionFunction("DeleteDetachedValues",
				"h", "${0.1}"));
		taskListModel.register(TaskEstimationFunction.createJuelExpressionFunction("KeepAnyValue",
				"h", "${0.1}"));

		taskListModel.register(new RepairModel());
		taskListModel.register(new TransformModel());

		Estimate estimate = taskListModel.evaluate(report, "h");
		estimate.print(System.out);

	}

}
