package de.uni_potsdam.hpi.isg.efes.pipeline.configuration;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.metrics.AverageStringLengthMetric;
import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.metrics.CharacterHistogramMetric;
import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.metrics.ConstancyMetric;
import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.metrics.FillStatusMetric;
import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.metrics.RangeOverlapMetric;
import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.metrics.TextPatternMetric;
import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.metrics.TopKValuesMetric;
import de.uni_potsdam.hpi.isg.efes.csg.effort.tasklist.TaskListModel;
import de.uni_potsdam.hpi.isg.efes.pipeline.configuration.model.Configuration;
import de.uni_potsdam.hpi.isg.efes.pipeline.configuration.model.MappingModelConfiguration;
import de.uni_potsdam.hpi.isg.efes.pipeline.configuration.model.RepairModelConfiguration;
import de.uni_potsdam.hpi.isg.efes.pipeline.configuration.model.SequentialConclusionModelConfiguration;
import de.uni_potsdam.hpi.isg.efes.pipeline.configuration.model.SimpleMappingModelConfiguration;
import de.uni_potsdam.hpi.isg.efes.pipeline.configuration.model.StructureFitModelConfiguration;
import de.uni_potsdam.hpi.isg.efes.pipeline.configuration.model.TaskListModelConfiguration;
import de.uni_potsdam.hpi.isg.efes.pipeline.configuration.model.TaskListModelConfiguration.EstimationFunction;
import de.uni_potsdam.hpi.isg.efes.pipeline.configuration.model.TaskListSubmodelConfiguration;
import de.uni_potsdam.hpi.isg.efes.pipeline.configuration.model.TransformModelConfiguration;
import de.uni_potsdam.hpi.isg.efes.pipeline.configuration.model.ValueFitModelConfiguration;

public class Configurations {

	public static Configuration loadConfiguration(File file) {
		return Configuration.load(file);
	}

	public static Configuration createExampleConfiguration() {
		Configuration configuration = new Configuration();

		// Configure data fit models.
		StructureFitModelConfiguration sfmc = new StructureFitModelConfiguration();

		ValueFitModelConfiguration vfmc = new ValueFitModelConfiguration();
		List<String> metricNames = new ArrayList<>();
		metricNames.add(FillStatusMetric.class.getCanonicalName());
		metricNames.add(AverageStringLengthMetric.class.getCanonicalName());
		metricNames.add(CharacterHistogramMetric.class.getCanonicalName());
		metricNames.add(ConstancyMetric.class.getCanonicalName());
		metricNames.add(RangeOverlapMetric.class.getCanonicalName());
		metricNames.add(TextPatternMetric.class.getCanonicalName());
		metricNames.add(TopKValuesMetric.class.getCanonicalName());
		vfmc.setMetricClassNames(metricNames);
		SequentialConclusionModelConfiguration conclusionModel = new SequentialConclusionModelConfiguration();
		vfmc.setConclusionModel(conclusionModel);

		MappingModelConfiguration mmc = new MappingModelConfiguration();

		configuration.setDataFitModelConfigurations(Arrays.asList(sfmc, vfmc, mmc));

		// Configure effort model.
		TaskListModelConfiguration tlmc = new TaskListModelConfiguration();
		tlmc.setProfile(TaskListModel.Profile.HIGH_QUALITY);
		List<TaskListSubmodelConfiguration> submodelConfigurations = new ArrayList<>();
		submodelConfigurations.add(new TransformModelConfiguration());
		submodelConfigurations.add(new RepairModelConfiguration());
		submodelConfigurations.add(new SimpleMappingModelConfiguration());
		tlmc.setSubmodels(submodelConfigurations);
		List<EstimationFunction> estimationFunctions = new ArrayList<>();
		estimationFunctions.add(new EstimationFunction("Aggregate Values", "min", "${3*repetitions}"));
		estimationFunctions.add(new EstimationFunction("Convert values", "min",
				"${(complexity < 3) ? (10*complexity) : (0.5*values)}"));
		estimationFunctions.add(new EstimationFunction("Generalize values", "min", "${0.2*distinctvalues}"));
		estimationFunctions.add(new EstimationFunction("Refine values", "min", "${0.2*values*complexity}"));
		estimationFunctions.add(new EstimationFunction("Drop values", "min", "${0.25*values}"));
		estimationFunctions.add(new EstimationFunction("Add values", "min", "${2*values}"));
		estimationFunctions.add(new EstimationFunction("Add Missing Unique Values", "min", "${2*repetitions}"));
		estimationFunctions.add(new EstimationFunction("Create Tuples For Detached Values", "min", "${15}"));
		estimationFunctions.add(new EstimationFunction("Keep Any Value", "min", "${15}"));
		estimationFunctions.add(new EstimationFunction("Delete Tuples With Missing Value", "min", "${5}"));
		estimationFunctions.add(new EstimationFunction("Delete Detached Values", "min", "${0}"));
		estimationFunctions.add(new EstimationFunction("Keep Any Value", "min", "${15}"));
		estimationFunctions.add(new EstimationFunction("Add Missing Tuples", "min", "${15}"));
		estimationFunctions.add(new EstimationFunction("Aggregate Tuples", "min", "${15}"));
		estimationFunctions.add(new EstimationFunction("Delete Dangling IND Values", "min", "${15}"));
		estimationFunctions.add(new EstimationFunction("Delete Dangling Tuples", "min", "${15}"));
		estimationFunctions.add(new EstimationFunction("Unlink all but one tuples", "min", "${15}"));
		estimationFunctions.add(new EstimationFunction("Add missing IND values", "min", "${15}"));
		estimationFunctions.add(new EstimationFunction("Add missing tuples", "min", "${15}"));
		estimationFunctions.add(new EstimationFunction("Aggregate tuples", "min", "${15}"));
		estimationFunctions.add(new EstimationFunction("Write mapping", "min",
				"${(5*fks)+(5*pk)+(3*meta)+(2*tables)}"));
		tlmc.setEstimationFunctions(estimationFunctions);

		configuration.setEffortModelConfiguration(tlmc);

		return configuration;
	}

}
