package de.uni_potsdam.hpi.isg.efes.pipeline.configuration.model;

import java.io.File;
import java.net.URISyntaxException;
import java.util.List;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElementWrapper;
import javax.xml.bind.annotation.XmlRootElement;

import de.uni_potsdam.hpi.isg.efes.csg.datafit.DataFitModel;
import de.uni_potsdam.hpi.isg.efes.csg.effort.EffortModel;
import de.uni_potsdam.hpi.isg.efes.pipeline.Pipeline;
import de.uni_potsdam.hpi.isg.efes.pipeline.configuration.Configurations;

@XmlRootElement
public class Configuration {

	private static final Class<?>[] boundClasses = new Class<?>[] { Configuration.class,
			StructureFitModelConfiguration.class, ValueFitModelConfiguration.class, MappingModelConfiguration.class,
			TaskListModelConfiguration.class, RepairModelConfiguration.class, TransformModelConfiguration.class,
			SimpleMappingModelConfiguration.class, SequentialConclusionModelConfiguration.class,
			SimpleEffortModelConfiguration.class, SimpleComplexityModelConfiguration.class };

	private List<DataFitModelConfiguration> dataFitModelConfigurations;
	private EffortModelConfiguration effortModelConfiguration;

	@XmlElement(name = "effortModel")
	public EffortModelConfiguration getEffortModelConfiguration() {
		return effortModelConfiguration;
	}

	public void setEffortModelConfiguration(EffortModelConfiguration effortModelConfiguration) {
		this.effortModelConfiguration = effortModelConfiguration;
	}

	@XmlElementWrapper(name = "dataFitModels")
	@XmlElement(name = "dataFitModel")
	public List<DataFitModelConfiguration> getDataFitModelConfigurations() {
		return dataFitModelConfigurations;
	}

	public void setDataFitModelConfigurations(List<DataFitModelConfiguration> dataFitModels) {
		this.dataFitModelConfigurations = dataFitModels;
	}

	public Pipeline createPipeline() {
		Pipeline pipeline = new Pipeline();

		for (DataFitModelConfiguration dataFitModelConfiguration : this.dataFitModelConfigurations) {
			DataFitModel dataFitModel = dataFitModelConfiguration.createDataFitModel();
			pipeline.addDataFitModel(dataFitModel);
		}

		EffortModel effortModel = this.effortModelConfiguration.createEffortModel();
		pipeline.setEffortModel(effortModel);

		return pipeline;
	}

	public void save(File file) {
		try {
			JAXBContext context = JAXBContext.newInstance(boundClasses);
			Marshaller marshaller = context.createMarshaller();
			marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
			marshaller.marshal(this, file);
		} catch (JAXBException e) {
			throw new RuntimeException(e);
		}
	}

	public static Configuration load(File file) {
		try {
			JAXBContext context = JAXBContext.newInstance(boundClasses);
			Unmarshaller unmarshaller = context.createUnmarshaller();
			return (Configuration) unmarshaller.unmarshal(file);
		} catch (JAXBException e) {
			throw new RuntimeException(e);
		}
	}

	public static void main(String[] args) throws JAXBException, URISyntaxException {
		Configuration configuration = Configurations.createExampleConfiguration();
		configuration.save(new File("example-config.xml"));
	}
}
