package de.uni_potsdam.hpi.isg.efes.pipeline.configuration.model;

import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlRootElement;

import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.conclusion.ConclusionModel;
import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.conclusion.SequentialConclusionModel;

@XmlRootElement
public class SequentialConclusionModelConfiguration extends ConclusionModelConfiguration {

	private double mediumComplexityBoundary = 0.5;
	private double lowComplexityBoundary = 0.8;
	private double noComplexityBoundary = 0.9;
	private double criticalFillStatusDiff = 0.3;
	private double minDomainConstancy = 0.7;
	private int minDomainEvidence = 5;
	
	@Override
	public ConclusionModel build() {
		SequentialConclusionModel model = new SequentialConclusionModel();
		model.setCriticalFillStatusDiff(this.criticalFillStatusDiff);
		model.setNoComplexityBoundary(noComplexityBoundary);
		model.setLowComplexityBoundary(lowComplexityBoundary);
		model.setMediumComplexityBoundary(mediumComplexityBoundary);
		model.setMinDomainConstancy(minDomainConstancy);
		model.setMinDomainEvidence(minDomainEvidence);
		return model;
	}

	@XmlAttribute(name = "mediumComplexity")
	public double getMediumComplexityBoundary() {
		return mediumComplexityBoundary;
	}

	public void setMediumComplexityBoundary(double mediumComplexityBoundary) {
		this.mediumComplexityBoundary = mediumComplexityBoundary;
	}

	@XmlAttribute(name = "lowComplexity")
	public double getLowComplexityBoundary() {
		return lowComplexityBoundary;
	}

	public void setLowComplexityBoundary(double lowComplexityBoundary) {
		this.lowComplexityBoundary = lowComplexityBoundary;
	}

	@XmlAttribute(name = "noComplexity")
	public double getNoComplexityBoundary() {
		return noComplexityBoundary;
	}

	public void setNoComplexityBoundary(double noComplexityBoundary) {
		this.noComplexityBoundary = noComplexityBoundary;
	}

	@XmlAttribute(name = "maxFillStatusDifference")
	public double getCriticalFillStatusDiff() {
		return criticalFillStatusDiff;
	}

	public void setCriticalFillStatusDiff(double criticalFillStatusDiff) {
		this.criticalFillStatusDiff = criticalFillStatusDiff;
	}

	@XmlAttribute(name = "minConstancyOfDomain")
	public double getMinDomainConstancy() {
		return minDomainConstancy;
	}

	public void setMinDomainConstancy(double minDomainConstancy) {
		this.minDomainConstancy = minDomainConstancy;
	}

	@XmlAttribute(name = "minTuplesForDomain")
	public int getMinDomainEvidence() {
		return minDomainEvidence;
	}

	public void setMinDomainEvidence(int minDomainEvidence) {
		this.minDomainEvidence = minDomainEvidence;
	}
	
	

}
