package de.uni_potsdam.hpi.isg.efes.pipeline.configuration.model;

import java.util.List;

import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElementWrapper;
import javax.xml.bind.annotation.XmlRootElement;

import de.uni_potsdam.hpi.isg.efes.csg.effort.EffortModel;
import de.uni_potsdam.hpi.isg.efes.csg.effort.tasklist.TaskEstimationFunction;
import de.uni_potsdam.hpi.isg.efes.csg.effort.tasklist.TaskListModel;

@XmlRootElement
public class TaskListModelConfiguration extends EffortModelConfiguration {

	private TaskListModel.Profile profile;

	private List<TaskListSubmodelConfiguration> submodels;

	private List<EstimationFunction> estimationFunctions;

	public TaskListModel.Profile getProfile() {
		return profile;
	}

	public void setProfile(TaskListModel.Profile profile) {
		this.profile = profile;
	}

	@XmlElementWrapper(name = "submodels")
	@XmlElement(name = "submodel")
	public List<TaskListSubmodelConfiguration> getSubmodels() {
		return submodels;
	}

	public void setSubmodels(List<TaskListSubmodelConfiguration> submodels) {
		this.submodels = submodels;
	}

	@XmlElementWrapper(name = "estimationFunctions")
	@XmlElement(name = "estimationFunction")
	public List<EstimationFunction> getEstimationFunctions() {
		return estimationFunctions;
	}

	public void setEstimationFunctions(List<EstimationFunction> estimationFunctions) {
		this.estimationFunctions = estimationFunctions;
	}

	@XmlRootElement
	public static class EstimationFunction {

		private String name;
		private String unit;
		private String expression;
		

		public EstimationFunction() {
			super();
		}

		public EstimationFunction(String name, String unit, String expression) {
			super();
			this.name = name;
			this.unit = unit;
			this.expression = expression;
		}

		public String getName() {
			return name;
		}

		public void setName(String name) {
			this.name = name;
		}

		public String getUnit() {
			return unit;
		}

		public void setUnit(String unit) {
			this.unit = unit;
		}

		public String getExpression() {
			return expression;
		}

		public void setExpression(String expression) {
			this.expression = expression;
		}

	}

	@Override
	public EffortModel createEffortModel() {
		TaskListModel taskListModel = new TaskListModel(this.profile);

		for (TaskListSubmodelConfiguration submodelConfiguration : this.submodels) {
			TaskListModel.Submodel submodel = submodelConfiguration.createSubmodel();
			taskListModel.register(submodel);
		}

		for (EstimationFunction estimationFunctionDescription : this.estimationFunctions) {
			TaskEstimationFunction function = TaskEstimationFunction.createJuelExpressionFunction(
					estimationFunctionDescription.getName(), estimationFunctionDescription.getUnit(),
					estimationFunctionDescription.getExpression());
			taskListModel.register(function);
		}

		return taskListModel;
	}

}
