package de.uni_potsdam.hpi.isg.efes.pipeline.configuration.model;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.List;

import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElementWrapper;
import javax.xml.bind.annotation.XmlRootElement;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.uni_potsdam.hpi.isg.efes.csg.datafit.DataFitModel;
import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.ValueCharacteristicsInspector;
import de.uni_potsdam.hpi.isg.efes.csg.datafit.values.metrics.ValueFitMetric;

@XmlRootElement
public class ValueFitModelConfiguration extends DataFitModelConfiguration {

	private static Logger logger = LogManager.getLogger(ValueFitModelConfiguration.class.getSimpleName());

	private List<String> metricClassNames;

	private ConclusionModelConfiguration conclusionModel;

	public ConclusionModelConfiguration getConclusionModel() {
		return conclusionModel;
	}

	public void setConclusionModel(ConclusionModelConfiguration conclusionModel) {
		this.conclusionModel = conclusionModel;
	}

	@XmlElementWrapper(name = "metricClassNames")
	@XmlElement(name = "metricClassName")
	public List<String> getMetricClassNames() {
		return metricClassNames;
	}

	public void setMetricClassNames(List<String> metricClassNames) {
		this.metricClassNames = metricClassNames;
	}

	@Override
	public DataFitModel createDataFitModel() {
		ValueCharacteristicsInspector model = new ValueCharacteristicsInspector();

		for (String metricClassName : this.metricClassNames) {
			metricClassName = metricClassName.trim();
			try {
				@SuppressWarnings("unchecked")
				Class<? extends ValueFitMetric> metricClass = (Class<? extends ValueFitMetric>) Class
						.forName(metricClassName);
				Constructor<? extends ValueFitMetric> constructor = metricClass.getConstructor();
				ValueFitMetric metric = constructor.newInstance();
				model.register(metric);
			} catch (ClassNotFoundException | ClassCastException | NoSuchMethodException | SecurityException
					| InstantiationException | IllegalAccessException | IllegalArgumentException
					| InvocationTargetException e) {
				logger.catching(e);
				logger.error("Could not load metric for class \"{}\".", metricClassName);
			}
		}

		model.setConclusionModel(conclusionModel.build());
		return model;
	}

}
