package de.uni_potsdam.hpi.isg.efes.csg.integration;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.uni_potsdam.hpi.isg.efes.csg.BasicSchemaGraph;
import de.uni_potsdam.hpi.isg.efes.csg.SchemaGraph;
import de.uni_potsdam.hpi.isg.efes.csg.constraints.ConstraintCollection;
import de.uni_potsdam.hpi.isg.efes.csg.constraints.NaturalNumberSet;
import de.uni_potsdam.hpi.isg.efes.csg.nodes.Node;
import de.uni_potsdam.hpi.isg.efes.csg.relationships.Relationship;

public class IntegratedSchemaGraph implements SchemaGraph {

	private static final Logger logger = LogManager.getLogger(IntegratedSchemaGraph.class.getSimpleName());

	private Map<String, BasicSchemaGraph> basicSchemaGraphs;

	public IntegratedSchemaGraph() {
		super();
		this.basicSchemaGraphs = new HashMap<String, BasicSchemaGraph>();
	}

	public void add(BasicSchemaGraph basicSchemaGraph) {
		this.basicSchemaGraphs.put(basicSchemaGraph.getName(), basicSchemaGraph);
	}

	@Override
	public ConstraintCollection getConstraints(Relationship relationship) {
		logger.trace("#getConstraints({})", relationship);
		String schemaName = BasicSchemaGraph.getSchemaNameFor(relationship);
		if (schemaName == null) {
			// If no schema name could be determined, we are dealing with a
			// relationship across instances. It is allegedly a union
			// relationship. However, we do not provide constraints across
			// instances, therefore the cardinality has to be inferred.
			return new ConstraintCollection(this, relationship);
		} else {
			BasicSchemaGraph basicSchemaGraph = this.basicSchemaGraphs.get(schemaName);
			if (basicSchemaGraph == null) {
				throw new IllegalArgumentException(String.valueOf(relationship) + ": No schema graph for schema name "
						+ schemaName + " found.");
			}
			return basicSchemaGraph.getConstraints(relationship);
		}
	}

	@Override
	public NaturalNumberSet getCardinality(Relationship relationship) {
		return getConstraints(relationship).calculateEffectiveCardinality();
	}

	@Override
	public boolean areOverlapping(Node node1, Node node2) {
		if (node1 == null || node2 == null) {
			logger.debug("One of the nodes is null.");
			return false;
		}

		if (node1.equals(node2)) {
			return true;
		}

		return node1.getContentType() == Node.ContentType.VALUE && node2.getContentType() == Node.ContentType.VALUE;
	}

	@Override
	public Map<String, Node> getAtomicNodes() {
		// TODO: Implement or modify interface.
		throw new UnsupportedOperationException();
	}

	@Override
	public Collection<Relationship> getAtomicRelationships(Node node) {
		// TODO: Implement or modify interface.
		throw new UnsupportedOperationException();
	}
	
	@Override
	public Collection<Relationship> getVirtualRelationships(Node node) {
		throw new UnsupportedOperationException();
	}

}
