package de.uni_potsdam.hpi.isg.efes.csg.integration;

import it.unimi.dsi.fastutil.PriorityQueue;
import it.unimi.dsi.fastutil.objects.ObjectArrayPriorityQueue;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

public abstract class WrappedIterator<E> implements Iterator<E> {

	private PriorityQueue<IteratorWithNext> queue = new ObjectArrayPriorityQueue<>();

	public WrappedIterator(List<Iterator<E>> iterators) {
		for (Iterator<E> iterator : iterators) {
			add(iterator);
		}
	}

	public void add(Iterator<E> iterator) {
		IteratorWithNext iteratorWithNext = new IteratorWithNext(iterator);
		if (iteratorWithNext.update()) {
			this.queue.enqueue(iteratorWithNext);
		}
	}

	@Override
	public boolean hasNext() {
		return !this.queue.isEmpty();
	}

	@Override
	public E next() {
		List<IteratorWithNext> iteratorsWithNext = new ArrayList<>(1);
		List<E> states = new ArrayList<>(1);

		// List<IteratorWithNext> x = new ArrayList<>();
		// while (!this.queue.isEmpty()) {
		// x.add(this.queue.dequeue());
		// }
		// logger.debug("{}", x);
		// for (IteratorWithNext iwn: x) {
		// this.queue.enqueue(iwn);
		// }

		IteratorWithNext iteratorWithNext = this.queue.dequeue();
		E firstE = iteratorWithNext.getNext();
		states.add(firstE);
		iteratorsWithNext.add(iteratorWithNext);

		while (!this.queue.isEmpty()) {
			iteratorWithNext = this.queue.first();
			E nextE = iteratorWithNext.getNext();
			if (shallGroup(firstE, nextE)) {
				iteratorsWithNext.add(iteratorWithNext);
				states.add(nextE);
				this.queue.dequeue();
			} else {
				break;
			}
		}

		E aggregate = aggregate(states);

		for (IteratorWithNext usedIteratorWithNext : iteratorsWithNext) {
			if (usedIteratorWithNext.update()) {
				this.queue.enqueue(usedIteratorWithNext);
			}
		}

		return aggregate;
	}

	protected abstract boolean shallGroup(E e1, E e2);

	protected abstract E aggregate(List<E> states);
	
	@Override
	public void remove() {
		throw new UnsupportedOperationException();
	}
	
	abstract protected int compare(E e1, E e2);
	
	private class IteratorWithNext implements Comparable<IteratorWithNext> {
		
		private Iterator<E> iterator;
		private E next;

		public IteratorWithNext(Iterator<E> iterator) {
			this.iterator = iterator;
		}

		@Override
		public int compareTo(IteratorWithNext o) {
			return WrappedIterator.this.compare(this.next, o.next);
		}

		public E getNext() {
			return next;
		}

		public boolean update() {
			if (!iterator.hasNext()) {
				return false;
			}
			next = iterator.next();
			return true;
		}

		@Override
		public String toString() {
			return "[next=" + next + "]";
		}

	}

}