package de.uni_potsdam.hpi.isg.efes.csg.mapping;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.uni_potsdam.hpi.isg.efes.csg.BasicSchemaGraph;
import de.uni_potsdam.hpi.isg.efes.csg.SchemaGraph;
import de.uni_potsdam.hpi.isg.efes.csg.instance.SchemaGraphInstance;
import de.uni_potsdam.hpi.isg.efes.csg.nodes.Node;
import de.uni_potsdam.hpi.isg.efes.csg.scenario.DataSet;

public class BasicNodeMapping implements NodeMapping {

	protected static final Logger logger = LogManager.getLogger(BasicNodeMapping.class.getSimpleName());

	protected final SchemaGraph schema;
	protected final SchemaGraphInstance instance;

	private final Map<Node, List<NodeCorrespondence>> correspondencesByTargetNode;

	@Deprecated
	public BasicNodeMapping(final DataSet source, final DataSet target) {
		this(source);
	}

	public BasicNodeMapping(final DataSet source) {
		this(source.getSchema(), source.getInstance());
	}

	public BasicNodeMapping(SchemaGraph schema, SchemaGraphInstance instance) {
		this.schema = schema;
		this.instance = instance;
		this.correspondencesByTargetNode = new HashMap<Node, List<NodeCorrespondence>>();
	}

	public BasicNodeMapping(NodeMapping mapping) {
		this(mapping.getSourceSchema(), mapping.getSourceInstance());
		this.correspondencesByTargetNode.putAll(mapping.getNodeCorrespondences());
	}

	public void addCorrespondence(final NodeCorrespondence correspondence) {
		if (correspondence == null)
			throw new IllegalArgumentException();

		List<NodeCorrespondence> correspondences = this.correspondencesByTargetNode.get(correspondence.getTargetNode());
		if (correspondences == null) {
			correspondences = new ArrayList<NodeCorrespondence>();
			this.correspondencesByTargetNode.put(correspondence.getTargetNode(), correspondences);
		} else if (correspondences.contains(correspondence)) {
			logger.warn("Tried to reinsert existing correspondence {}.", correspondence);
		}

		correspondences.add(correspondence);

	}

	public Collection<NodeCorrespondence> getCorrespondencesForTargetNode(Node targetNode) {
		Collection<NodeCorrespondence> correspondences = this.correspondencesByTargetNode.get(targetNode);
		return correspondences == null ? Collections.<NodeCorrespondence> emptyList() : correspondences;
	}

	public void removeCorrespondencesForTargetNode(Node targetNode) {
		this.correspondencesByTargetNode.remove(targetNode);
	}

	public Map<Node, List<NodeCorrespondence>> getNodeCorrespondences() {
		return Collections.unmodifiableMap(this.correspondencesByTargetNode);
	}

	public boolean isBackedByInstance() {
		return getSourceInstance() != null;
	}

	@Override
	public SchemaGraphInstance getSourceInstance() {
		return this.instance;
	}

	@Override
	public SchemaGraph getSourceSchema() {
		return this.schema;
	}

	@Override
	public String toString() {
		return "BasicNodeMapping [schema=" + schema + ", instance=" + instance + ", #correspondencesByTargetNode="
				+ correspondencesByTargetNode.size() + "]";
	}

	public static BasicNodeMapping createIdentityMapping(DataSet dataSet) {
		BasicNodeMapping mapping = new BasicNodeMapping(dataSet, dataSet);
		BasicSchemaGraph schema = dataSet.getSchema();
		for (Node node : schema.getAtomicNodes().values()) {
			mapping.addCorrespondence(new NodeCorrespondence(node, node));
		}
		return mapping;
	}

	public static BasicNodeMapping createIdentityMapping(DataSet sourceDataSet, DataSet targetDataSet) {
		BasicNodeMapping mapping = new BasicNodeMapping(sourceDataSet, targetDataSet);
		BasicSchemaGraph targetSchema = targetDataSet.getSchema();
		BasicSchemaGraph sourceSchema = sourceDataSet.getSchema();
		for (Node targetNode : targetSchema.getAtomicNodes().values()) {
			if (targetNode.isMetaData()) {
				continue;
			}
			String nodeName = BasicSchemaGraph.getSchemalessNameFor(targetNode);
			String sourceNodeName = sourceSchema.createSchemaSpecificName(nodeName);
			Node sourceNode = sourceSchema.getNode(sourceNodeName);
			if (sourceNode == null) {
				logger.warn("Could not find an identical source node for {}.", sourceNodeName);
			} else {
				mapping.addCorrespondence(new NodeCorrespondence(sourceNode, targetNode));
			}
		}
		return mapping;
	}

}
