package de.uni_potsdam.hpi.isg.efes.csg.mapping;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.uni_potsdam.hpi.isg.efes.csg.BasicSchemaGraph;
import de.uni_potsdam.hpi.isg.efes.csg.constraints.NaturalNumberBag;
import de.uni_potsdam.hpi.isg.efes.csg.instance.DataStoreAdapter;
import de.uni_potsdam.hpi.isg.efes.csg.instance.DataStoreAdapters;
import de.uni_potsdam.hpi.isg.efes.csg.instance.SchemaGraphInstance;
import de.uni_potsdam.hpi.isg.efes.csg.nodes.Node;
import de.uni_potsdam.hpi.isg.efes.csg.nodes.UnionNode;
import de.uni_potsdam.hpi.isg.efes.csg.relationships.Relationship;

public class MappedDataStoreAdapter implements DataStoreAdapter {

	private static final Logger logger = LogManager.getLogger(MappedDataStoreAdapter.class.getSimpleName());

	private RelationshipMapping mapping;

	private BasicSchemaGraph schemaGraph;

	@Override
	public void setSchemaGraph(BasicSchemaGraph schemaGraph) {
		this.schemaGraph = schemaGraph;
	}

	@Override
	public NaturalNumberBag queryCardinality(Relationship relationship) {
		logger.debug("Querying cardinality of {}.", relationship);

		if (!this.schemaGraph.contains(relationship)) {
			throw new IllegalArgumentException(String.valueOf(relationship));
		}

		NaturalNumberBag cardinality = this.mapping.getRealSourceCardinality(relationship);
		logger.debug("Result: {}.", cardinality);
		return cardinality;
	}

	@Override
	public void setDataStore(Object dataStore) throws IllegalArgumentException {
		if (dataStore == null || !(dataStore instanceof MappedDataStore)) {
			throw new IllegalArgumentException(String.valueOf(dataStore));
		}

		MappedDataStore mappedDataStore = (MappedDataStore) dataStore;
		SchemaGraphInstance sourceInstance = mappedDataStore.mapping.getSourceInstance();
		this.mapping = mappedDataStore.mapping;

		if (!sourceInstance.hasDataStoreAdapter()) {
			throw new IllegalArgumentException(sourceInstance
					+ " has no data store adapter and can thus not be used as mapped data store.");
		}
	}

	@Override
	public int querySize(Node node) {
		if (!this.schemaGraph.containsNode(node)) {
			throw new IllegalArgumentException(String.valueOf(node));
		}

		Collection<NodeCorrespondence> correspondences = this.mapping.getCorrespondencesForTargetNode(node);

		if (correspondences.isEmpty()) {
			return 0;
		}
		
		List<Node> sourceNodes = new ArrayList<>();
		for (NodeCorrespondence correspondence : correspondences) {
			sourceNodes.add(correspondence.getSourceNode());
		}
		Node queryNode = UnionNode.union(sourceNodes);

		SchemaGraphInstance sourceInstance = mapping.getSourceInstance();
		int size = sourceInstance.getRealSize(queryNode);
		return size;
	}

	@Override
	public Iterator<KeyValueCount> queryKeysWithCardinality(Relationship relationship) {
		throw new UnsupportedOperationException();
	}

	@Override
	public Iterator<KeyValueCount> queryKeyValuesWithCardinality(Relationship relationship) {
		throw new UnsupportedOperationException();
	}

	@Override
	public Iterator<String> getValues(Node node) {
		throw new UnsupportedOperationException();
	}

	public static class MappedDataStore {

		static {
			// Automatically register if this class is used.
			DataStoreAdapters.registerFactory(MappedDataStore.class, new Factory());
		}

		private RelationshipMapping mapping;

		public MappedDataStore(RelationshipMapping mapping) {
			super();
			this.mapping = mapping;
		}

	}

	public static class Factory implements DataStoreAdapter.Factory {

		@Override
		public DataStoreAdapter createAdapter() {
			return new MappedDataStoreAdapter();
		}

	}

}
