package de.uni_potsdam.hpi.isg.efes.csg.mapping;

import java.util.Collection;
import java.util.List;
import java.util.Map;

import de.uni_potsdam.hpi.isg.efes.csg.constraints.NaturalNumberBag;
import de.uni_potsdam.hpi.isg.efes.csg.constraints.NaturalNumberSet;
import de.uni_potsdam.hpi.isg.efes.csg.relationships.Relationship;

public interface RelationshipMapping extends NodeMapping {

	Collection<RelationshipCorrespondence> getCorrespondencesForTargetRelationship(Relationship targetRelationship);

	/**
	 * Translate a atomic source relationship to a complex target relationship. This is useful when a single target relationship corresponds to multiple source
	 * relationships.<br>
	 * Currently only support atomic relationships.
	 * 
	 * @return a single relationship or <code>null</code> if no source relationship exists
	 */
	Relationship translateTargetRelationship(Relationship targetRelationship);

	NaturalNumberSet estimateSourceCardinality(Relationship targetRelationship);

	NaturalNumberBag getRealSourceCardinality(Relationship targetRelationship);

	Map<Relationship, List<RelationshipCorrespondence>> getRelationshipCorrespondences();

	/**
	 * In a valid {@link RelationshipMapping}, each {@link NodeCorrespondence} must be covered by at least one {@link RelationshipCorrespondence} wrt. to domain
	 * and codomain. This method validates the integrity of this condition.
	 * 
	 * @throws IllegalStateException if the mapping is not valid.
	 */
	void validateDomainsAndCodomains() throws IllegalStateException;

}
