package de.uni_potsdam.hpi.isg.efes.csg.scenario;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.uni_potsdam.hpi.isg.efes.csg.BasicSchemaGraph;
import de.uni_potsdam.hpi.isg.efes.csg.instance.SchemaGraphInstance;
import de.uni_potsdam.hpi.isg.efes.csg.instance.VirtualSchemaGraphInstance;
import de.uni_potsdam.hpi.isg.efes.csg.util.RelationalSchemaToCSGConverter;
import de.uni_potsdam.hpi.isg.utils.db.Database;
import de.uni_potsdam.hpi.isg.utils.db.metamodel.RelationalSchema;
import de.uni_potsdam.hpi.isg.utils.db.metamodel.SchemaExtractor;

/**
 * A data set has a schema and data as well.
 * 
 * @author Sebastian Kruse
 */
public class DataSet {
	
	private static final Logger logger = LogManager.getLogger(DataSet.class.getSimpleName());

	public static enum Type {
		RELATIONAL
	}

	private BasicSchemaGraph schema;

	private Object dataStore;
	
	private SchemaGraphInstance instance;

	private Type type;

	public DataSet(BasicSchemaGraph schema, Type type, Object dataStore) {
		if (schema == null && dataStore == null)
			throw new IllegalArgumentException();

		this.type = type;
		this.schema = schema;
		this.dataStore = dataStore;
	}

	public BasicSchemaGraph getSchema() {
		if (this.schema == null)
			try {
				loadSchemaFromDataStore();
			} catch (Exception e) {
				throw new RuntimeException(e);
			}
		return schema;
	}
	
	public SchemaGraphInstance getInstance() {
		if (this.instance == null) {
			VirtualSchemaGraphInstance instance = new VirtualSchemaGraphInstance(this.schema);

			if (this.dataStore == null || !instance.acquireDataStoreAdapter(this.dataStore)) {
				logger.debug("Could not associate inspected schema graph to a data set.");
			} else {
				this.instance = instance;
			}

		}
		return instance;
	}

	public Object getDataStore() {
		return dataStore;
	}
	
	public Type getType() {
		return type;
	}

	public void loadSchemaFromDataStore() throws Exception {
		if (this.type == Type.RELATIONAL && dataStore instanceof Database<?>) {
			// Extract the relational schema from the database.

			Database<?> database = (Database<?>) dataStore;
			SchemaExtractor<?> schemaExtractor = database.getSchemaExtractor();
			RelationalSchema relationalSchema = schemaExtractor.extractSchema(null);

			// Convert the schema into a schema graph.
			RelationalSchemaToCSGConverter converter = new RelationalSchemaToCSGConverter();
			this.schema = converter.convert(relationalSchema);

		} else {
			throw new IllegalStateException("Do not know how to load schema from data store: " + dataStore);
		}

	}

	@Override
	public String toString() {
		return "DataSet [type=" + type + ", schema=" + schema + ", dataStore=" + dataStore + "]";
	}

}
