package de.uni_potsdam.hpi.isg.efes.csg.scenario;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import de.uni_potsdam.hpi.isg.efes.csg.mapping.BasicNodeMapping;

/**
 * An integration scenario consists of target and source schema graphs and
 * mappings in between them. Then, reasoning can be done in such a scenario.
 * 
 * @author Sebastian Kruse
 */
public class IntegrationScenario {
	
	private List<DataSet> sourceDataSets;
	
	private DataSet targetDataSet;
	
	private Map<DataSet, BasicNodeMapping> mappings;

	public IntegrationScenario(DataSet targetDataSet) {
		super();
		this.targetDataSet = targetDataSet;
		this.sourceDataSets = new ArrayList<DataSet>();
		this.mappings = new HashMap<DataSet, BasicNodeMapping>();
	}

	public void addSource(DataSet source, BasicNodeMapping mapping) {
		if (source == null || mapping == null) {
			throw new IllegalArgumentException();
		}
		if (this.sourceDataSets.contains(source)) {
			throw new IllegalArgumentException();
		}
		
		this.sourceDataSets.add(source);
		this.mappings.put(source, mapping);
	}

	public Map<DataSet, BasicNodeMapping> getMappings() {
		return mappings;
	}

	public DataSet getTargetDataSet() {
		return targetDataSet;
	}
	
	public List<DataSet> getSourceDataSets() {
		return Collections.unmodifiableList(sourceDataSets);
	}

	public void setMapping(DataSet sourceDataSet, BasicNodeMapping mapping) {
		if (!this.sourceDataSets.contains(sourceDataSet)) {
			throw new IllegalArgumentException("Not a source data set: "+ sourceDataSet);
		}
		this.mappings.put(sourceDataSet, mapping);
	}

}
