package de.uni_potsdam.hpi.isg.efes.csg.constraints;

import java.util.ArrayList;
import java.util.Collection;

import de.uni_potsdam.hpi.isg.efes.csg.relationships.Relationship;

/**
 * Currently constraint collections only aggregate constraints on an
 * identical relation. In the future, however, they should form a
 * hierarchical relation on the constrained relationships.
 * 
 * @author Sebastian Kruse
 */
public class ConstraintCollection {

	private Relationship relationship;
	private Collection<NaturalNumberSet> cardinalities;
	private ConstraintStorage storage;

	public ConstraintCollection(ConstraintStorage storage, Relationship relationship) {
		this.relationship = relationship;
		this.cardinalities = new ArrayList<>(1);
		this.storage = storage;
	}

	public void add(NaturalNumberSet cardinality) {
		this.cardinalities.add(cardinality);
	}

	public NaturalNumberSet getInferredConstraint() {
		return relationship.inferCardinality(this.storage);
	}

	public Constraint toConstraint() {
		return new ConstraintImpl(relationship,
				calculateEffectiveCardinality());
	}

	public NaturalNumberSet calculateEffectiveCardinality() {
		NaturalNumberSet result = getInferredConstraint();
		for (NaturalNumberSet cardinality : cardinalities) {
			result = result.intersect(cardinality);
		}
		return result;
	}

	public Collection<NaturalNumberSet> getCardinalities() {
		return cardinalities;
	}

	public Relationship getConstrainedRelationship() {
		return relationship;
	}

	@Override
	public String toString() {
		return "ConstraintCollection [relationship=" + relationship
				+ ", cardinalities=" + cardinalities + "]";
	}
	
}