package de.uni_potsdam.hpi.isg.efes.csg.constraints;

import it.unimi.dsi.fastutil.ints.Int2IntMap;
import it.unimi.dsi.fastutil.ints.Int2IntOpenHashMap;
import it.unimi.dsi.fastutil.ints.Int2IntRBTreeMap;
import it.unimi.dsi.fastutil.ints.IntIterator;
import it.unimi.dsi.fastutil.ints.IntSet;

public class NaturalNumberBag {

	public static final NaturalNumberBag EMPTY_BAG = new NaturalNumberBag();
	private Int2IntMap countsPerNumber;
	private int cardinality;

	public NaturalNumberBag() {
		// this.countsPerNumber = new Int2IntOpenHashMap();
		this.countsPerNumber = new Int2IntRBTreeMap();
		this.countsPerNumber.defaultReturnValue(0);
		this.cardinality = 0;
	}

	public NaturalNumberBag(NaturalNumberBag bag) {
		this.countsPerNumber = new Int2IntOpenHashMap(bag.countsPerNumber);
		this.countsPerNumber.defaultReturnValue(0);
		this.cardinality = bag.cardinality;
	}

	public void count(int number, int delta) {
		int oldCount = this.countsPerNumber.get(number);
		int newCount = oldCount + delta;
		if (newCount < 0) {
			throw new IllegalArgumentException("Cannot add " + number + "=>"+ delta + " to "+ this+".");
		} else if (newCount == 0) {
			this.countsPerNumber.remove(number);
		} else {
			this.countsPerNumber.put(number, newCount);
		}
		this.cardinality += delta;
	}

	public void count(int number) {
		count(number, 1);
	}

	public int getCountFor(int number) {
		return this.countsPerNumber.get(number);
	}

	public IntSet getCountedCardinalities() {
		return this.countsPerNumber.keySet();
	}

	public Int2IntMap getCountsPerNumber() {
		return countsPerNumber;
	}

	public void unionAll(NaturalNumberBag other) {
		for (Int2IntMap.Entry entry : other.countsPerNumber.int2IntEntrySet()) {
			this.count(entry.getIntKey(), entry.getIntValue());
		}
	}

	/**
	 * Approximates this bag as good as possible by using the minimum and maximum value of this bag.
	 * @return the interval or an empty set if this bag is empty
	 */
	public NaturalNumberSet toInterval() {

		IntSet keySet = this.countsPerNumber.keySet();
		if (keySet.isEmpty())
			return NaturalNumberSet.EMPTY_SET;

		// This code only works if there are no 0-counts in the map.
		IntIterator iterator = keySet.iterator();
		int i = iterator.nextInt();
		int max = i, min = i;
		while (iterator.hasNext()) {
			i = iterator.nextInt();
			min = Math.min(i, min);
			max = Math.max(i, max);
		}

		return new NaturalNumberInterval(min, max);
	}

	public int getCardinality() {
		return this.cardinality;
	}

	@Override
	public String toString() {
		return countsPerNumber.toString();
	}

	public int getSum() {
		int sum = 0;
		for (Int2IntMap.Entry entry : this.countsPerNumber.int2IntEntrySet()) {
			sum += entry.getIntKey() * entry.getIntValue();
		}
		return sum;
	}

}
