package de.uni_potsdam.hpi.isg.efes.csg.constraints;

import de.uni_potsdam.hpi.isg.efes.csg.util.PartialOrder;

public interface NaturalNumberSet {

	static NaturalNumberSet ONE = new NaturalNumberInterval(1, 1);
	static NaturalNumberSet ZERO = new NaturalNumberInterval(0, 0);
	static NaturalNumberSet GREATER_OR_EQUAL_ONE = new NaturalNumberInterval(1);
	static NaturalNumberSet GREATER_THAN_ONE = new NaturalNumberInterval(2);
	static NaturalNumberSet ZERO_OR_ONE = new NaturalNumberInterval(0, 1);
	static NaturalNumberSet NATURAL_NUMBERS = new NaturalNumberInterval(0);

	static int INFINITE = -1;

	boolean isEmpty();

	NaturalNumberSet add(NaturalNumberSet other);

	NaturalNumberSet addInterval(NaturalNumberInterval interval);

	NaturalNumberSet inclExclAdd(NaturalNumberSet other);

	NaturalNumberSet inclExclAddInterval(NaturalNumberInterval other);

	NaturalNumberSet compose(NaturalNumberSet other);

	NaturalNumberSet composeInterval(NaturalNumberInterval interval);

	NaturalNumberSet intersect(NaturalNumberSet other);

	NaturalNumberSet intersectInterval(NaturalNumberInterval other);

	NaturalNumberSet union(NaturalNumberSet other);

	NaturalNumberSet unionInterval(NaturalNumberInterval other);

	boolean hasElement(int element);

	int getMin();

	int getMax();

	boolean isSubsetOf(NaturalNumberSet other);

	boolean isSubsetOfInterval(NaturalNumberInterval other);

	boolean isSupersetOf(NaturalNumberSet other);

	boolean isSupersetOfInterval(NaturalNumberInterval other);

	static PartialOrder<NaturalNumberSet> SUBSET_ORDER = new PartialOrder<NaturalNumberSet>() {

		public de.uni_potsdam.hpi.isg.efes.csg.util.PartialOrder.Type compare(NaturalNumberSet a, NaturalNumberSet b) {
			if (a.isSubsetOf(b)) {
				if (b.equals(a)) {
					return PartialOrder.Type.EQUAL;
				}
				return PartialOrder.Type.SMALLER;
			} else if (b.isSubsetOf(a)) {
				return PartialOrder.Type.GREATER;
			}
			return PartialOrder.Type.UNORDERED;
		}
	};

	static NaturalNumberSet EMPTY_SET = new NaturalNumberSet() {

		public boolean isEmpty() {
			return true;
		};

		public boolean hasElement(int element) {
			return false;
		};

		@Override
		public boolean isSubsetOf(NaturalNumberSet other) {
			return true;
		}

		public boolean isSubsetOfInterval(NaturalNumberInterval other) {
			return true;
		};

		public boolean isSupersetOf(NaturalNumberSet other) {
			return other.isEmpty();
		};

		public boolean isSupersetOfInterval(NaturalNumberInterval other) {
			return false;
		};

		@Override
		public NaturalNumberSet intersectInterval(NaturalNumberInterval other) {
			return this;
		}

		@Override
		public NaturalNumberSet intersect(NaturalNumberSet other) {
			return this;
		}

		@Override
		public NaturalNumberSet union(NaturalNumberSet other) {
			return other;
		}

		@Override
		public NaturalNumberSet unionInterval(NaturalNumberInterval other) {
			return other;
		}

		@Override
		public int getMin() {
			throw new UnsupportedOperationException();
		}

		@Override
		public int getMax() {
			throw new UnsupportedOperationException();
		}

		@Override
		public NaturalNumberSet composeInterval(NaturalNumberInterval interval) {
			return this;
		}

		@Override
		public NaturalNumberSet compose(NaturalNumberSet other) {
			return this;
		}

		@Override
		public NaturalNumberSet addInterval(NaturalNumberInterval interval) {
			return this;
		}

		@Override
		public NaturalNumberSet add(NaturalNumberSet other) {
			return this;
		}

		@Override
		public NaturalNumberSet inclExclAdd(NaturalNumberSet other) {
			return this;
		}

		@Override
		public NaturalNumberSet inclExclAddInterval(NaturalNumberInterval other) {
			return this;
		}

		public String toString() {
			return "{}";
		};
	};
}
