package de.uni_potsdam.hpi.isg.efes.csg.instance;

import java.util.Iterator;

import de.uni_potsdam.hpi.isg.efes.csg.BasicSchemaGraph;
import de.uni_potsdam.hpi.isg.efes.csg.constraints.NaturalNumberBag;
import de.uni_potsdam.hpi.isg.efes.csg.nodes.Node;
import de.uni_potsdam.hpi.isg.efes.csg.relationships.Relationship;

/**
 * Exposes the underlying data store of a {@link VirtualSchemaGraphInstance}.
 * 
 * @author Sebastian Kruse
 */
public interface DataStoreAdapter {

	/**
	 * Associate the {@link BasicSchemaGraph} with this {@link DataStoreAdapter}
	 * .
	 * 
	 * @param schemaGraph
	 */
	void setSchemaGraph(BasicSchemaGraph schemaGraph);

	/**
	 * Query the cardinality of the given relationship. This adapter should be
	 * associated with a schema graph before.
	 * 
	 * @param relationship
	 * @see #setSchemaGraph(BasicSchemaGraph)
	 */
	NaturalNumberBag queryCardinality(Relationship relationship);

	void setDataStore(Object dataStore) throws IllegalArgumentException;

	int querySize(Node node);

	/**
	 * Optional operation.<br>
	 * Returns an iterator that enumerates all the values from the
	 * relationship's source node and the number of its occurrences in the
	 * relationship.
	 * 
	 * @return
	 */
	Iterator<KeyValueCount> queryKeysWithCardinality(Relationship relationship);

	public static interface KeyValueCount {

		String getKey();

		String getValue();

		int getCount();

		void setCount(int sumCount);
		
		boolean keyValueEqual(KeyValueCount kc);

	}

	public static interface Factory {

		DataStoreAdapter createAdapter();

	}

	Iterator<KeyValueCount> queryKeyValuesWithCardinality(Relationship relationship);

	Iterator<String> getValues(Node node);

}
