package de.uni_potsdam.hpi.isg.efes.csg.instance;

import java.util.HashMap;
import java.util.Map;

import de.uni_potsdam.hpi.isg.efes.csg.BasicSchemaGraph;
import de.uni_potsdam.hpi.isg.utils.db.PostgresqlDatabase;

public class DataStoreAdapters {

	private static final Map<Class<?>, DataStoreAdapter.Factory> registeredFactories = new HashMap<>();
	static {
		registerFactory(PostgresqlDatabase.class, new PostgresqlAdapter.Factory());
	}
	
	public static void registerFactory(Class<?> dataStoreClass, DataStoreAdapter.Factory adapterFactory) {
		registeredFactories.put(dataStoreClass, adapterFactory);
	}

	public static DataStoreAdapter createFor(BasicSchemaGraph schema, Object dataStore) {
		Class<? extends Object> dataStoreClass = dataStore.getClass();
		DataStoreAdapter.Factory factory = registeredFactories.get(dataStoreClass);
		if (factory == null) {
			throw new IllegalArgumentException("No data store adapter factory registered for " + dataStoreClass);
		}
		DataStoreAdapter adapter = factory.createAdapter();
		adapter.setDataStore(dataStore);
		adapter.setSchemaGraph(schema);
		return adapter;
	}
	
	
	
}
