package de.uni_potsdam.hpi.isg.efes.csg.instance;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Iterator;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

public abstract class JDBCStatementIterator<E> implements Iterator<E> {
	
	private static Logger logger = LogManager.getLogger(JDBCStatementIterator.class.getSimpleName());
	
	private Statement stmt;
	private ResultSet resultSet;
	private E next = null;
	private boolean isInitialized = false;

	public JDBCStatementIterator(Statement stmt, String sqlQuery) throws SQLException {
		this.stmt = stmt;
		this.resultSet = stmt.executeQuery(sqlQuery);
	}

	private void moveToNext() {
		try {
			if (this.resultSet.next()) {
				this.next = readNext(this.resultSet);
			} else {
				this.next = null;
			}
		} catch (SQLException e) {
			close();
			throw new RuntimeException(e);
		}
		this.isInitialized = true;
	}

	protected abstract E readNext(ResultSet resultSet) throws SQLException;

	private void close() {
		try {
			this.stmt.close();
		} catch (SQLException e1) {
			logger.catching(e1);
		}
		this.next = null;
	}

	@Override
	public boolean hasNext() {
		if (!isInitialized) moveToNext();
		return this.next != null;
	}

	@Override
	public E next() {
		if (!isInitialized) moveToNext();
		E result = this.next;
		moveToNext();
		return result;
	}

	@Override
	public void remove() {
		throw new UnsupportedOperationException();
	}
}
