package de.uni_potsdam.hpi.isg.efes.csg.nodes;

import java.util.Collection;
import java.util.Collections;

public abstract class AbstractNode implements Node {

	protected final String name;

	private boolean isMetaData = false;

	public AbstractNode(String name) {
		this.name = name;
	}

	public String getName() {
		return name;
	}

	@Override
	public boolean isSimpleIdentityNode() {
		return getContentType() == Node.ContentType.IDENTITY && getOperationType() == Node.Operation.NONE;
	}

	@Override
	public boolean isSimpleValueNode() {
		return getContentType() == Node.ContentType.VALUE && getOperationType() == Node.Operation.NONE;
	}

	@Override
	public boolean isMetaData() {
		return this.isMetaData;
	}

	@Override
	public void setIsMetaData(boolean isMetaData) {
		this.isMetaData = isMetaData;
	}

	@Override
	public Collection<Node> getParentNodes() {
		// Implementation for atomic nodes.
		if (this.getOperationType() != Node.Operation.NONE) {
			throw new RuntimeException("This function must be overriden by " + getClass() + ".");
		}
		return Collections.<Node>singleton(this);
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((name == null) ? 0 : name.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		AbstractNode other = (AbstractNode) obj;
		if (name == null) {
			if (other.name != null)
				return false;
		} else if (!name.equals(other.name))
			return false;
		return true;
	}


	@Override
	public int compareTo(Node o) {
		return this.getName().compareTo(o.getName());
	}

}
