package de.uni_potsdam.hpi.isg.efes.csg.nodes;

import java.util.List;

import de.uni_potsdam.hpi.isg.efes.csg.relationships.CoalesceRelationship;

/**
 * 
 * @author Sebastian Kruse
 */
public class CrossProductNode extends AbstractNode {

	private final List<Node> parentNodes;
	private final CoalesceRelationship projectingRelationship;
	private ContentType contentType;

	public CrossProductNode(String name, List<Node> parentNodes, CoalesceRelationship constructingRelationship) {
		super(name);
		this.parentNodes = parentNodes;
		this.projectingRelationship = constructingRelationship;
		this.contentType = Node.ContentType.EMPTY;
		for (Node parentNode : parentNodes) {
			this.contentType = this.contentType.mix(parentNode.getContentType());
		}
	}

	@Override
	public String getName() {
		return name;
	}

	public CoalesceRelationship getProjectingRelationship() {
		return projectingRelationship;
	}

	@Override
	public List<Node> getParentNodes() {
		return parentNodes;
	}

	public boolean contains(Node node) {
		return this.equals(node) || this.parentNodes.contains(node);
	}

	@Override
	public ContentType getContentType() {
		return this.contentType;
	}

	@Override
	public Operation getOperationType() {
		return Node.Operation.CROSS_PRODUCT;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((projectingRelationship == null) ? 0 : projectingRelationship.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		CrossProductNode other = (CrossProductNode) obj;
		if (projectingRelationship == null) {
			if (other.projectingRelationship != null)
				return false;
		} else if (!projectingRelationship.equals(other.projectingRelationship))
			return false;
		return true;
	}

	@Override
	public String toString() {
		return "[" + this.name + "]";
	}

}
