package de.uni_potsdam.hpi.isg.efes.csg.relationships;

import java.util.Collection;
import java.util.Collections;

import de.uni_potsdam.hpi.isg.efes.csg.nodes.Node;

public abstract class AbstractAtomicRelationship extends AbstractRelationship {

	private Node startNode, endNode;

	public AbstractAtomicRelationship(Node startNode, Node endNode) {
		if (!allowNullNodes() && (startNode == null || endNode == null))
			throw new IllegalArgumentException("Start node and end node must not be null.");
		this.startNode = startNode;
		this.endNode = endNode;
	}

	protected AbstractAtomicRelationship(Node startNode, Node endNode, Relationship inverse) {
		super(inverse);
		if (!allowNullNodes() && (startNode == null || endNode == null))
			throw new IllegalArgumentException("Start node and end node must not be null.");
		this.startNode = startNode;
		this.endNode = endNode;
	}

	public boolean contains(Node node) {
		return this.getStartNode().equals(node) || this.getEndNode().equals(node);
	}

	@Override
	public boolean isAtomic() {
		return true;
	}

	@Override
	public int getComplexity() {
		return 1;
	}
	
	protected boolean allowNullNodes() {
		return false;
	}

	@Override
	public Collection<Relationship> getAtomicRelationships() {
		return Collections.<Relationship> singleton(this);
	}

	@Override
	public Node getStartNode() {
		return this.startNode;
	}

	@Override
	public Node getEndNode() {
		return this.endNode;
	}
	
	@Override
	public OperationType getOperationType() {
		return Relationship.OperationType.NONE;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((endNode == null) ? 0 : endNode.hashCode());
		result = prime * result + ((startNode == null) ? 0 : startNode.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		AbstractAtomicRelationship other = (AbstractAtomicRelationship) obj;
		if (endNode == null) {
			if (other.endNode != null)
				return false;
		} else if (!endNode.equals(other.endNode))
			return false;
		if (startNode == null) {
			if (other.startNode != null)
				return false;
		} else if (!startNode.equals(other.startNode))
			return false;
		return true;
	}

}
