package de.uni_potsdam.hpi.isg.efes.csg.relationships;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

public abstract class AbstractComplexRelationship extends AbstractRelationship {

	private Collection<Relationship> atomicRelationships;

	public AbstractComplexRelationship() {
		super();
	}

	public AbstractComplexRelationship(Relationship inverse) {
		super(inverse);
	}

	@Override
	public boolean isAtomic() {
		return getComplexity() == 1;
	}

	@Override
	public boolean isExplicit() {
		for (Relationship relationship : getAtomicRelationships()) {
			if (relationship.isExplicit())
				return true;
		}
		return false;
	}

	@Override
	public int getComplexity() {
		return getAtomicRelationships().size();
	}

	@Override
	public Collection<Relationship> getAtomicRelationships() {
		if (this.atomicRelationships == null)
			this.atomicRelationships = collectAtomicRelationships();
		return this.atomicRelationships;
	}

	protected abstract Collection<Relationship> collectAtomicRelationships();

	protected static List<Relationship> invertAll(Collection<Relationship> relationships) {
		List<Relationship> inverse = new ArrayList<>(relationships.size());
		for (Relationship relationship : relationships) {
			inverse.add(relationship.getInverse());
		}
		return inverse;
	}

}
