package de.uni_potsdam.hpi.isg.efes.csg.relationships;

import java.util.Collection;
import java.util.Collections;

import de.uni_potsdam.hpi.isg.efes.csg.constraints.ConstraintStorage;
import de.uni_potsdam.hpi.isg.efes.csg.constraints.NaturalNumberSet;
import de.uni_potsdam.hpi.isg.efes.csg.nodes.Node;

public class EmptyRelationship extends AbstractAtomicRelationship {

	public static EmptyRelationship createWithStartNode(Node startNode) {
		return new EmptyRelationship(startNode, null);
	}
	
	public static EmptyRelationship createWithEndNode(Node endNode) {
		return new EmptyRelationship(null, endNode);
	}
	
	private EmptyRelationship(Node startNode, Node endNode) {
		super(startNode, endNode);
	}

	@Override
	protected boolean allowNullNodes() {
		return true;
	}
	
	@Override
	public boolean contains(Node node) {
		return node.equals(getStartNode()) || node.equals(getEndNode());
	}

	@Override
	public boolean isAtomic() {
		throw new UnsupportedOperationException();
	}

	@Override
	public NaturalNumberSet inferCardinality(ConstraintStorage constraintStorage) {
		if (getStartNode() == null)
			return NaturalNumberSet.EMPTY_SET;
		return NaturalNumberSet.ZERO;
	}

	@Override
	protected AbstractAtomicRelationship createInverse() {
		return new EmptyRelationship(getEndNode(), getStartNode());
	}

	@Override
	public int getComplexity() {
		return 0;
	}

	@Override
	public Collection<Relationship> getAtomicRelationships() {
		return Collections.emptyList();
	}

	public String toString() {
		return getStartNode() + "->" + getEndNode();
	}

	@Override
	public ContentType getContentType() {
		return Relationship.ContentType.EMPTY;
	}

	@Override
	public Relationship compose(Relationship relationship) {
		throw new UnsupportedOperationException();
	}
	
	

}
