package de.uni_potsdam.hpi.isg.efes.csg.relationships;

import java.util.Collection;

import de.uni_potsdam.hpi.isg.efes.csg.constraints.ConstraintStorage;
import de.uni_potsdam.hpi.isg.efes.csg.constraints.NaturalNumberSet;
import de.uni_potsdam.hpi.isg.efes.csg.nodes.Node;

/**
 * Represents an edge between two (possible virtual) nodes in the schema graph.
 * 
 * @author Sebastian Kruse
 */
public interface Relationship {

	static enum ContentType {
		EXPLICIT, EQUALITY, EMPTY;

		public static ContentType mix(ContentType o1, ContentType o2) {
			switch (o1) {
			case EMPTY:
				return o2;
			case EXPLICIT:
				return EXPLICIT;
			default:
				break;
			}

			switch (o2) {
			case EMPTY:
				return o2;
			default:
				break;
			}

			if (o1 != o2) {
				return EXPLICIT;
			}
			return o1;
		}

		public ContentType mix(ContentType other) {
			return ContentType.mix(this, other);
		}
	}

	static enum OperationType {
		NONE, COALESCE, FORK, COMPOSITE, UNION, COMPOUND, IDENTITY
	}

	/**
	 * Tells whether the given node is somehow contained in the relationship as
	 * start or end node.<br>
	 * <i>Note that implementations might only work for atomic nodes correctly.</i>
	 */
	boolean contains(Node node);

	ContentType getContentType();

	OperationType getOperationType();

	/**
	 * If a constraint storage does not provide a constraint for a relationship,
	 * this method tries to determine the relationships cardinality as good as
	 * possible with <b>OTHER</b> constraints.
	 * 
	 * @param constraintStorage
	 *            is the constraint storage that provides those other
	 *            constraints
	 * @return the inferred cardinality
	 */
	public NaturalNumberSet inferCardinality(ConstraintStorage constraintStorage);

	boolean isAtomic();

	boolean isExplicit();

	boolean isEmpty();

	Collection<Relationship> getAtomicRelationships();

	int getComplexity();

	Node getStartNode();

	Node getEndNode();

	Relationship compose(Relationship relationship);

	Relationship getInverse();

}
