package de.uni_potsdam.hpi.isg.efes.csg.search;

import de.uni_potsdam.hpi.isg.efes.csg.constraints.Constraint;
import de.uni_potsdam.hpi.isg.efes.csg.constraints.NaturalNumberSet;
import de.uni_potsdam.hpi.isg.efes.csg.relationships.Relationship;
import de.uni_potsdam.hpi.isg.efes.csg.util.PartialOrder;

/**
 * Compares two constraints a and b by their specificity and conciseness. A
 * smaller constraint is intended to be simple and more useful.<br>
 * a is smaller than b if:
 * <ol>
 * <li>a's cardinality is a subset b's cardinality</li>
 * <li>or their cardinalities are equal but a's complexity is smaller than b's</li>
 * </ol> 
 * 
 * @see NaturalNumberSet#SUBSET_ORDER
 * @see Relationship#getComplexity()
 *  
 * @author Sebastian Kruse
 */
public class ConstraintOrder implements PartialOrder<Constraint> {
	
	public static final ConstraintOrder INSTANCE = new ConstraintOrder(); 

	@Override
	public PartialOrder.Type compare(Constraint a, Constraint b) {
		PartialOrder.Type cardinalityOrder = NaturalNumberSet.SUBSET_ORDER.compare(
				a.getCardinality(), b.getCardinality());
		switch (cardinalityOrder) {
		case EQUAL:
			Relationship relationshipA = a.getRelationship();
			Relationship relationshipB = b.getRelationship();
			int complexityComparison = Integer.compare(relationshipA.getComplexity(),
					relationshipB.getComplexity());
			if (complexityComparison < 0) {
				return PartialOrder.Type.SMALLER;
			} else if (complexityComparison > 0) {
				return PartialOrder.Type.GREATER;
			} else {
				return PartialOrder.Type.EQUAL;
			}
			
		default:
			return cardinalityOrder;
		}
	}

}
