package de.uni_potsdam.hpi.isg.efes.csg.util;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.apache.logging.log4j.LogManager;

public class PartialOrderQueue<Element> {

	private LinkedList<Element> store = new LinkedList<Element>();
	private PartialOrder<Element> order;

	public PartialOrderQueue(PartialOrder<Element> order) {
		this.order = order;
	}

	public void remove(Element e) {
		for (Iterator<Element> i = this.store.iterator(); i.hasNext();) {
			if (e.equals(i.next())) {
				i.remove();
			}
		}
	}

	public void add(Element e) {
		int targetIndex = 0;
		for (Element existingElement : this.store) {
			PartialOrder.Type ordering = this.order.compare(e, existingElement);
			if (ordering == PartialOrder.Type.SMALLER) {
				break;
			}
			targetIndex++;
		}
		this.store.add(targetIndex, e);
	}

	public void addAll(Collection<Element> collection) {
		for (Element e : collection) {
			add(e);
		}
	}

	public Element poll() {
		return this.store.pollLast();
	}

	public boolean isEmpty() {
		return this.store.isEmpty();
	}

	public void clear() {
		this.store.clear();
	}
	
	public void retainSmallest() {
		for (int i = 0; i < this.store.size() - 1; i++) {
			Element baseElement = this.store.get(i);
			for (int j = i + 1; j < this.store.size(); ) {
				Element checkedElement = this.store.get(j);
				if (this.order.compare(baseElement, checkedElement) == PartialOrder.Type.SMALLER) {
					LogManager.getLogger().debug("Removing {} in favor of {}", checkedElement, baseElement);
					this.store.remove(j);
				} else {
					j++;
				}
			}
		}
	}
	
	public List<Element> toList() {
		return new ArrayList<>(this.store);
	}
}