package de.uni_potsdam.hpi.isg.efes.csg.constraints;


import org.junit.Assert;
import org.junit.Test;

public class NaturalNumberIntervalTest {
	
	@Test
	public void testAdd() {
		NaturalNumberSet a = new NaturalNumberInterval(0, 1);
		NaturalNumberSet b = new NaturalNumberInterval(2, 5);
		
		NaturalNumberSet result = a.add(b);
		Assert.assertEquals(2, result.getMin());
		Assert.assertEquals(6, result.getMax());
	}
	
	@Test
	public void testAddWithOpenSet() {
		NaturalNumberSet a = new NaturalNumberInterval(0, 1);
		NaturalNumberSet b = new NaturalNumberInterval(1);
		
		NaturalNumberSet result = a.add(b);
		Assert.assertEquals(1, result.getMin());
		Assert.assertEquals(NaturalNumberSet.INFINITE, result.getMax());
	}
	
	@Test
	public void testCompose() {
		NaturalNumberSet a = new NaturalNumberInterval(0, 1);
		NaturalNumberSet b = new NaturalNumberInterval(1, 1);
		NaturalNumberSet result = a.compose(b);
		Assert.assertEquals(0, result.getMin());
		Assert.assertEquals(1, result.getMax());
		
		a = new NaturalNumberInterval(0, 1);
		b = new NaturalNumberInterval(1);
		result = a.compose(b);
		Assert.assertEquals(0, result.getMin());
		Assert.assertEquals(NaturalNumberSet.INFINITE, result.getMax());
		
		a = new NaturalNumberInterval(0, 1);
		b = new NaturalNumberInterval(0, 1);
		result = a.compose(b);
		Assert.assertEquals(0, result.getMin());
		Assert.assertEquals(1, result.getMax());

		a = new NaturalNumberInterval(1, 1);
		b = new NaturalNumberInterval(1, 1);
		result = a.compose(b);
		Assert.assertEquals(1, result.getMin());
		Assert.assertEquals(1, result.getMax());
		
		a = new NaturalNumberInterval(1, 1);
		b = new NaturalNumberInterval(1);
		result = a.compose(b);
		Assert.assertEquals(1, result.getMin());
		Assert.assertEquals(NaturalNumberSet.INFINITE, result.getMax());
		
		a = new NaturalNumberInterval(1, 1);
		b = new NaturalNumberInterval(0, 1);
		result = a.compose(b);
		Assert.assertEquals(0, result.getMin());
		Assert.assertEquals(1, result.getMax());
		
		a = new NaturalNumberInterval(1);
		b = new NaturalNumberInterval(1, 1);
		result = a.compose(b);
		Assert.assertEquals(1, result.getMin());
		Assert.assertEquals(NaturalNumberSet.INFINITE, result.getMax());
		
		b = new NaturalNumberInterval(1);
		result = a.compose(b);
		Assert.assertEquals(1, result.getMin());
		Assert.assertEquals(NaturalNumberSet.INFINITE, result.getMax());
		
		b = new NaturalNumberInterval(0, 1);
		result = a.compose(b);
		Assert.assertEquals(0, result.getMin());
		Assert.assertEquals(NaturalNumberSet.INFINITE, result.getMax());
	}
	
	@Test
	public void testIntersect() {
		NaturalNumberSet a = new NaturalNumberInterval(0, 1);
		NaturalNumberSet b = new NaturalNumberInterval(1, 1);
		NaturalNumberSet result = a.intersect(b);
		Assert.assertEquals(1, result.getMin());
		Assert.assertEquals(1, result.getMax());
		
		a = new NaturalNumberInterval(0, 1);
		b = new NaturalNumberInterval(1);
		result = a.intersect(b);
		Assert.assertEquals(1, result.getMin());
		Assert.assertEquals(1, result.getMax());
		
		a = new NaturalNumberInterval(0, 1);
		b = new NaturalNumberInterval(0, 1);
		result = a.intersect(b);
		Assert.assertEquals(0, result.getMin());
		Assert.assertEquals(1, result.getMax());
		
		a = new NaturalNumberInterval(1, 1);
		b = new NaturalNumberInterval(1, 1);
		result = a.intersect(b);
		Assert.assertEquals(1, result.getMin());
		Assert.assertEquals(1, result.getMax());
		
		a = new NaturalNumberInterval(1, 1);
		b = new NaturalNumberInterval(1);
		result = a.intersect(b);
		Assert.assertEquals(1, result.getMin());
		Assert.assertEquals(1, result.getMax());
		
		a = new NaturalNumberInterval(1, 1);
		b = new NaturalNumberInterval(0, 1);
		result = a.intersect(b);
		Assert.assertEquals(1, result.getMin());
		Assert.assertEquals(1, result.getMax());
		
		a = new NaturalNumberInterval(1);
		b = new NaturalNumberInterval(1, 1);
		result = a.intersect(b);
		Assert.assertEquals(1, result.getMin());
		Assert.assertEquals(1, result.getMax());
		
		b = new NaturalNumberInterval(1);
		result = a.intersect(b);
		Assert.assertEquals(1, result.getMin());
		Assert.assertEquals(NaturalNumberSet.INFINITE, result.getMax());
		
		b = new NaturalNumberInterval(0, 1);
		result = a.intersect(b);
		Assert.assertEquals(1, result.getMin());
		Assert.assertEquals(1, result.getMax());
	}
	
	@Test
	public void testIsSubsetOf() {
		Assert.assertTrue(NaturalNumberSet.ONE.isSubsetOf(NaturalNumberSet.NATURAL_NUMBERS));
		Assert.assertTrue(NaturalNumberSet.ONE.isSubsetOf(NaturalNumberSet.GREATER_OR_EQUAL_ONE));
		Assert.assertTrue(NaturalNumberSet.ONE.isSubsetOf(NaturalNumberSet.ZERO_OR_ONE));
		Assert.assertTrue(NaturalNumberSet.ONE.isSubsetOf(NaturalNumberSet.ONE));
		
		Assert.assertTrue(NaturalNumberSet.ZERO_OR_ONE.isSubsetOf(NaturalNumberSet.NATURAL_NUMBERS));
		Assert.assertTrue(!NaturalNumberSet.ZERO_OR_ONE.isSubsetOf(NaturalNumberSet.GREATER_OR_EQUAL_ONE));
		Assert.assertTrue(NaturalNumberSet.ZERO_OR_ONE.isSubsetOf(NaturalNumberSet.ZERO_OR_ONE));
		Assert.assertTrue(!NaturalNumberSet.ZERO_OR_ONE.isSubsetOf(NaturalNumberSet.ONE));

		Assert.assertTrue(NaturalNumberSet.GREATER_OR_EQUAL_ONE.isSubsetOf(NaturalNumberSet.NATURAL_NUMBERS));
		Assert.assertTrue(NaturalNumberSet.GREATER_OR_EQUAL_ONE.isSubsetOf(NaturalNumberSet.GREATER_OR_EQUAL_ONE));
		Assert.assertTrue(!NaturalNumberSet.GREATER_OR_EQUAL_ONE.isSubsetOf(NaturalNumberSet.ZERO_OR_ONE));
		Assert.assertTrue(!NaturalNumberSet.GREATER_OR_EQUAL_ONE.isSubsetOf(NaturalNumberSet.ONE));
		
		Assert.assertTrue(NaturalNumberSet.NATURAL_NUMBERS.isSubsetOf(NaturalNumberSet.NATURAL_NUMBERS));
		Assert.assertTrue(!NaturalNumberSet.NATURAL_NUMBERS.isSubsetOf(NaturalNumberSet.GREATER_OR_EQUAL_ONE));
		Assert.assertTrue(!NaturalNumberSet.NATURAL_NUMBERS.isSubsetOf(NaturalNumberSet.ZERO_OR_ONE));
		Assert.assertTrue(!NaturalNumberSet.NATURAL_NUMBERS.isSubsetOf(NaturalNumberSet.ONE));
	}
	
	@Test
	public void testUnion() {
		NaturalNumberSet result = NaturalNumberSet.ZERO.union(NaturalNumberSet.NATURAL_NUMBERS);
		Assert.assertEquals(result, NaturalNumberSet.NATURAL_NUMBERS);

		result = NaturalNumberSet.NATURAL_NUMBERS.union(NaturalNumberSet.ZERO);
		Assert.assertEquals(result, NaturalNumberSet.NATURAL_NUMBERS);
		
		result = NaturalNumberSet.ZERO.union(NaturalNumberSet.GREATER_OR_EQUAL_ONE);
		Assert.assertEquals(result, NaturalNumberSet.NATURAL_NUMBERS);
		
		result = NaturalNumberSet.GREATER_OR_EQUAL_ONE.union(NaturalNumberSet.ZERO);
		Assert.assertEquals(result, NaturalNumberSet.NATURAL_NUMBERS);
		
		result = NaturalNumberSet.ZERO.union(NaturalNumberSet.ONE);
		Assert.assertEquals(result, NaturalNumberSet.ZERO_OR_ONE);
		
		result = NaturalNumberSet.ONE.union(NaturalNumberSet.ZERO);
		Assert.assertEquals(result, NaturalNumberSet.ZERO_OR_ONE);
		
		
		
	}

}
