package de.uni_potsdam.hpi.isg.efes.csg.instance;

import java.io.File;
import java.io.FileReader;
import java.net.URL;
import java.sql.SQLException;
import java.util.Collection;
import java.util.Properties;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.junit.Test;

import de.uni_potsdam.hpi.isg.efes.csg.BasicSchemaGraph;
import de.uni_potsdam.hpi.isg.efes.csg.constraints.Constraint;
import de.uni_potsdam.hpi.isg.efes.csg.constraints.NaturalNumberBag;
import de.uni_potsdam.hpi.isg.efes.csg.nodes.Node;
import de.uni_potsdam.hpi.isg.efes.csg.util.RelationalSchemaToCSGConverter;
import de.uni_potsdam.hpi.isg.utils.db.Database;
import de.uni_potsdam.hpi.isg.utils.db.PostgresqlDatabase;
import de.uni_potsdam.hpi.isg.utils.db.metamodel.PostgresqlSchemaExtractor;
import de.uni_potsdam.hpi.isg.utils.db.metamodel.RelationalSchema;

public class PostgresqlAdapterTest {

	@Test
	public void test() throws SQLException {

		Properties properties = loadProperties();

		PostgresqlDatabase db = (PostgresqlDatabase) Database.newInstance(properties);
		db.open();
		PostgresqlSchemaExtractor extractor = new PostgresqlSchemaExtractor(db);
		RelationalSchema schema = extractor.extractSchema("public");
		db.close();

		RelationalSchemaToCSGConverter converter = new RelationalSchemaToCSGConverter();
		BasicSchemaGraph schemaGraph = converter.convert(schema);

		DataStoreAdapter dataStoreAdapter = new PostgresqlAdapter();
		dataStoreAdapter.setSchemaGraph(schemaGraph);
		dataStoreAdapter.setDataStore(db);
		Node artistNode = schemaGraph.getNode("artist");
		Node annotationTextNode = schemaGraph.getNode("annotation.text");
		Collection<Constraint> artist2annotationTextConstraints = schemaGraph.findCardinality(
				artistNode, annotationTextNode);
		Logger rootLogger = LogManager.getRootLogger();
		for (Constraint c : artist2annotationTextConstraints) {
			NaturalNumberBag cardinality = dataStoreAdapter.queryCardinality(c.getRelationship());
			rootLogger.debug(cardinality);
		}
//		for (BinaryRelationship relationship : schemaGraph.getAllRelationships()) {
//			NaturalNumberBag cardinality = dataStoreAdapter.queryCardinality(relationship);
//			rootLogger.debug(cardinality);
//			rootLogger.debug("");
//		}

	}

	private Properties loadProperties() {
		final URL propertiesURL = this.getClass().getResource("/postgres.properties");
		final File propertiesFile = new File(propertiesURL.getFile());
		final Properties properties = new Properties();
		try (FileReader reader = new FileReader(propertiesFile)) {
			properties.load(reader);
		} catch (Exception e) {
			throw new RuntimeException("Test defect.", e);
		}
		return properties;
	}

}
