package de.uni_potsdam.hpi.isg.efes.csg.relationships;

import java.util.ArrayList;
import java.util.List;

import org.junit.Assert;
import org.junit.Test;

import de.uni_potsdam.hpi.isg.efes.csg.BasicSchemaGraph;
import de.uni_potsdam.hpi.isg.efes.csg.constraints.NaturalNumberInterval;
import de.uni_potsdam.hpi.isg.efes.csg.constraints.NaturalNumberSet;
import de.uni_potsdam.hpi.isg.efes.csg.nodes.Datatype;
import de.uni_potsdam.hpi.isg.efes.csg.nodes.IdentityNode;
import de.uni_potsdam.hpi.isg.efes.csg.nodes.ValueNode;
import de.uni_potsdam.hpi.isg.efes.csg.relationships.Relationship;
import de.uni_potsdam.hpi.isg.efes.csg.relationships.UnionRelationship;

public class UnionRelationshipTest {
	
	@Test
	public void testInferCardinality() {
		BasicSchemaGraph schemaGraph = new BasicSchemaGraph("test");
		
		IdentityNode artist = new IdentityNode("test.artist");
		schemaGraph.add(artist);

		ValueNode artistName = new ValueNode("test.artist.name", new Datatype.String(10));
		schemaGraph.add(artistName);
		schemaGraph.addExplicitRelationships(artist, artistName);
		schemaGraph.addConstraint(artist, artistName, NaturalNumberSet.ONE);
		schemaGraph.addConstraint(artistName, artist, NaturalNumberSet.ONE);
		
		ValueNode artistRealname = new ValueNode("test.artist.realname", new Datatype.String(10));
		schemaGraph.add(artistRealname);
		schemaGraph.addExplicitRelationships(artist, artistRealname);
		schemaGraph.addConstraint(artist, artistRealname, NaturalNumberSet.NATURAL_NUMBERS);
		schemaGraph.addConstraint(artistRealname, artist, NaturalNumberSet.GREATER_OR_EQUAL_ONE);

		IdentityNode band = new IdentityNode("test.band");
		schemaGraph.add(band);
		
		ValueNode bandName = new ValueNode("test.band.name", new Datatype.String(10));
		schemaGraph.add(bandName);
		schemaGraph.addExplicitRelationships(band, bandName);
		schemaGraph.addConstraint(band, bandName, NaturalNumberSet.ONE);
		schemaGraph.addConstraint(bandName, band, NaturalNumberSet.ONE);
		
		List<Relationship> relationships = new ArrayList<>();
		relationships.add(schemaGraph.getAtomicRelationship(artist, artistName));
		relationships.add(schemaGraph.getAtomicRelationship(artist, artistRealname));
		relationships.add(schemaGraph.getAtomicRelationship(band, bandName));

		Relationship unionRelationship = UnionRelationship.union(relationships);
		NaturalNumberSet inferredCardinality = schemaGraph.getCardinality(unionRelationship);
		Assert.assertEquals(NaturalNumberSet.GREATER_OR_EQUAL_ONE, inferredCardinality);
		
		unionRelationship = unionRelationship.getInverse();
		inferredCardinality = schemaGraph.getCardinality(unionRelationship);
		Assert.assertEquals(NaturalNumberSet.GREATER_OR_EQUAL_ONE, inferredCardinality);
		
		relationships.clear();
		relationships.add(schemaGraph.getAtomicRelationship(artist, artistName));
		relationships.add(schemaGraph.getAtomicRelationship(band, bandName));

		unionRelationship = new UnionRelationship(relationships);
		inferredCardinality = schemaGraph.getCardinality(unionRelationship);
		Assert.assertEquals(NaturalNumberSet.ONE, inferredCardinality);

		unionRelationship = unionRelationship.getInverse();
		inferredCardinality = schemaGraph.getCardinality(unionRelationship);
		Assert.assertEquals(new NaturalNumberInterval(1, 2), inferredCardinality);
	}

}
