package de.uni_potsdam.hpi.isg.efes.csg.search;

import java.util.Collection;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.junit.Assert;
import org.junit.Test;

import de.uni_potsdam.hpi.isg.efes.csg.BasicSchemaGraph;
import de.uni_potsdam.hpi.isg.efes.csg.constraints.NaturalNumberSet;
import de.uni_potsdam.hpi.isg.efes.csg.nodes.Node;
import de.uni_potsdam.hpi.isg.efes.csg.util.RelationalSchemaToCSGConverter;
import de.uni_potsdam.hpi.isg.utils.db.metamodel.Column;
import de.uni_potsdam.hpi.isg.utils.db.metamodel.Constraint;
import de.uni_potsdam.hpi.isg.utils.db.metamodel.ForeignKeyConstraint;
import de.uni_potsdam.hpi.isg.utils.db.metamodel.NotNullConstraint;
import de.uni_potsdam.hpi.isg.utils.db.metamodel.RelationalSchema;
import de.uni_potsdam.hpi.isg.utils.db.metamodel.Table;
import de.uni_potsdam.hpi.isg.utils.db.metamodel.UniqueConstraint;
import de.uni_potsdam.hpi.isg.utils.util.CollectionUtil;

public class PartialOrderGraphSearcherTest {

	private static Logger logger = LogManager.getLogger("tests");
	
	@Test
	public void testCorrectlyInvolvesVirtualRelationships() {

		RelationalSchema schema = new RelationalSchema("test");
		Table telephone = schema.addTable("telephone");
		Column prefix = telephone.addColumn("prefix");
		Column number = telephone.addColumn("number");

		Constraint prefixNN = new NotNullConstraint(telephone, prefix);
		schema.addConstraint(prefixNN);
		Constraint numberNN = new NotNullConstraint(telephone, number);
		schema.addConstraint(numberNN);
		Constraint telephonePK = new UniqueConstraint(telephone, prefix, number);
		schema.addConstraint(telephonePK);

		Table customer = schema.addTable("customer");
		Column cPrefix = customer.addColumn("prefix");
		Column cNumber = customer.addColumn("number");

		Constraint cPrefixNN = new NotNullConstraint(customer, cPrefix);
		schema.addConstraint(cPrefixNN);
		Constraint cNumberNN = new NotNullConstraint(customer, cNumber);
		schema.addConstraint(cNumberNN);
		Constraint customerPK = new UniqueConstraint(customer, cPrefix, cNumber);
		schema.addConstraint(customerPK);

		Constraint fk = new ForeignKeyConstraint(customer, new Column[] { cPrefix, cNumber }, telephone, new Column[] {
				prefix, number });
		schema.addConstraint(fk);
		
		RelationalSchemaToCSGConverter converter = new RelationalSchemaToCSGConverter();
		BasicSchemaGraph schemaGraph = converter.convert(schema);
		
		Node customerNode = schemaGraph.getNode(RelationalSchemaToCSGConverter.createTableNodeName(schemaGraph, customer));
		Node telephoneNode = schemaGraph.getNode(RelationalSchemaToCSGConverter.createTableNodeName(schemaGraph, telephone));
		Collection<de.uni_potsdam.hpi.isg.efes.csg.constraints.Constraint> constraints = schemaGraph.findCardinality(customerNode, telephoneNode);
		
		logger.debug("Found constraints: {}", constraints);
		
		Assert.assertEquals(1, constraints.size());
		Assert.assertEquals(6, CollectionUtil.getAny(constraints).getRelationship().getComplexity());
		Assert.assertEquals(NaturalNumberSet.ONE, CollectionUtil.getAny(constraints).getCardinality());
		
	}
}
