package de.uni_potsdam.hpi.isg.efes.spicy;

import java.util.ArrayList;
import java.util.List;

/**
 * Describes a subset of the configuration files of ++Spicy tool.
 * 
 * @author Sebastian Kruse
 */
public class SpicyConfiguration {

	private RelationalDataset source, target;
	private List<Correspondence> correspondences;

	public RelationalDataset getSource() {
		return source;
	}

	void setSource(RelationalDataset source) {
		this.source = source;
	}

	public RelationalDataset getTarget() {
		return target;
	}

	void setTarget(RelationalDataset target) {
		this.target = target;
	}

	public List<Correspondence> getCorrespondences() {
		return correspondences;
	}

	void setCorrespondences(List<Correspondence> correspondences) {
		this.correspondences = correspondences;
	}

	@Override
	public String toString() {
		return "SpicyConfiguration [source=" + source + ", target=" + target + ", correspondences="
				+ correspondences + "]";
	}

	public static RelationalPath resolveRelationalPath(String path) {
		String[] elements = path.split("\\.");
		if (elements.length != 4)
			throw new IllegalArgumentException("Illegal path pattern: " + path);

		return new RelationalPath(elements[0], elements[1], elements[3]);
	}

	public static class RelationalPath {

		private String dataSet, table, attribute;

		public RelationalPath(String dataSet, String table, String attribute) {
			super();
			if (dataSet.isEmpty() || table.isEmpty() || attribute.isEmpty()) {
				throw new IllegalArgumentException();
			}
			
			this.dataSet = dataSet;
			this.table = table;
			this.attribute = attribute;
		}

		public String getDataSet() {
			return dataSet;
		}

		public String getTable() {
			return table;
		}

		public String getAttribute() {
			return attribute;
		}

	}

	public static class RelationalDataset {

		String dbDriver, dbUri, dbLogin, dbPassword;

		public String getDbDriver() {
			return dbDriver;
		}

		void setDbDriver(String dbDriver) {
			this.dbDriver = dbDriver;
		}

		public String getDbUri() {
			return dbUri;
		}

		void setDbUri(String dbUri) {
			this.dbUri = dbUri;
		}

		public String getDbLogin() {
			return dbLogin;
		}

		void setDbLogin(String dbLogin) {
			this.dbLogin = dbLogin;
		}

		public String getDbPassword() {
			return dbPassword;
		}

		void setDbPassword(String dbPassword) {
			this.dbPassword = dbPassword;
		}

		@Override
		public String toString() {
			return "RelationalDataset [dbDriver=" + dbDriver + ", dbUri=" + dbUri + ", dbLogin="
					+ dbLogin + ", dbPassword=" + dbPassword + "]";
		}

		@Override
		public int hashCode() {
			final int prime = 31;
			int result = 1;
			result = prime * result + ((dbDriver == null) ? 0 : dbDriver.hashCode());
			result = prime * result + ((dbLogin == null) ? 0 : dbLogin.hashCode());
			result = prime * result + ((dbPassword == null) ? 0 : dbPassword.hashCode());
			result = prime * result + ((dbUri == null) ? 0 : dbUri.hashCode());
			return result;
		}

		@Override
		public boolean equals(Object obj) {
			if (this == obj)
				return true;
			if (obj == null)
				return false;
			if (getClass() != obj.getClass())
				return false;
			RelationalDataset other = (RelationalDataset) obj;
			if (dbDriver == null) {
				if (other.dbDriver != null)
					return false;
			} else if (!dbDriver.equals(other.dbDriver))
				return false;
			if (dbLogin == null) {
				if (other.dbLogin != null)
					return false;
			} else if (!dbLogin.equals(other.dbLogin))
				return false;
			if (dbPassword == null) {
				if (other.dbPassword != null)
					return false;
			} else if (!dbPassword.equals(other.dbPassword))
				return false;
			if (dbUri == null) {
				if (other.dbUri != null)
					return false;
			} else if (!dbUri.equals(other.dbUri))
				return false;
			return true;
		}

	}

	public static class Correspondence {

		private List<String> sourcePaths = new ArrayList<String>();
		private String targetPath, transformationFunction;
		private double confidence;

		public List<String> getSourcePaths() {
			return sourcePaths;
		}

		void addSourcePath(String sourcePath) {
			this.sourcePaths.add(sourcePath);
		}

		public String getTargetPath() {
			return targetPath;
		}

		void setTargetPath(String targetPath) {
			this.targetPath = targetPath;
		}

		public String getTransformationFunction() {
			return transformationFunction;
		}

		void setTransformationFunction(String transformationFunction) {
			this.transformationFunction = transformationFunction;
		}

		public double getConfidence() {
			return confidence;
		}

		void setConfidence(double confidence) {
			this.confidence = confidence;
		}

		@Override
		public String toString() {
			return "Correspondence [sourcePaths=" + sourcePaths + ", targetPath=" + targetPath
					+ ", transformationFunction=" + transformationFunction + ", confidence="
					+ confidence + "]";
		}

		@Override
		public int hashCode() {
			final int prime = 31;
			int result = 1;
			long temp;
			temp = Double.doubleToLongBits(confidence);
			result = prime * result + (int) (temp ^ (temp >>> 32));
			result = prime * result + ((sourcePaths == null) ? 0 : sourcePaths.hashCode());
			result = prime * result + ((targetPath == null) ? 0 : targetPath.hashCode());
			result = prime * result
					+ ((transformationFunction == null) ? 0 : transformationFunction.hashCode());
			return result;
		}

		@Override
		public boolean equals(Object obj) {
			if (this == obj)
				return true;
			if (obj == null)
				return false;
			if (getClass() != obj.getClass())
				return false;
			Correspondence other = (Correspondence) obj;
			if (Double.doubleToLongBits(confidence) != Double.doubleToLongBits(other.confidence))
				return false;
			if (sourcePaths == null) {
				if (other.sourcePaths != null)
					return false;
			} else if (!sourcePaths.equals(other.sourcePaths))
				return false;
			if (targetPath == null) {
				if (other.targetPath != null)
					return false;
			} else if (!targetPath.equals(other.targetPath))
				return false;
			if (transformationFunction == null) {
				if (other.transformationFunction != null)
					return false;
			} else if (!transformationFunction.equals(other.transformationFunction))
				return false;
			return true;
		}

	}

}
