package de.uni_potsdam.hpi.isg.utils.datatypes;

import java.nio.charset.Charset;
import java.util.Arrays;
import java.util.Comparator;

public class Utf8String implements Cloneable, Comparable<Utf8String> {
	
	public static final Charset UTF8_CHARSET = Charset.forName("UTF-8");
	private static final boolean USE_HASH_CACHING = false;

	public static byte[] toBytes(String s) {
		return s.getBytes(UTF8_CHARSET);
	}
	
	public static String toString(byte[] bytes) {
		return new String(bytes, UTF8_CHARSET);
	}
	
	public static Comparator<byte[]> BYTE_ARRAY_COMPARATOR = new Comparator<byte[]>() {

		@Override
		public int compare(byte[] a1, byte[] a2) {
			if (a1 == null)
				return a2 == null ? 0 : 1;
			if (a2 == null)
				return -1;
			int minLength = Math.min(a1.length, a2.length);
			for (int i = 0; i < minLength; i++) {
				int diff = a1[i] - a2[i];
				if (diff != 0) {
					return diff;
				}
			}
			return a1.length - a2.length;		
		}
	};
	
	public static Utf8String createUtf8For(String s) {
		Utf8String instance = new Utf8String();
		instance.setDataUtf8Encoded(s);
		return instance;
	}

	public static Utf8String createFor(byte[] data) {
		Utf8String instance = createInstance();
		instance.setData(data);
		return instance;
	}
	
	private static Utf8String createInstance() {
		if (USE_HASH_CACHING) {
			return new HashChachingByteString();
		}
		return new Utf8String();
	}
	
	private byte[] data; 

	private Utf8String() { }
	
	public void setData(byte[] data) {
		this.data = data;
	}
	
	public void setDataUtf8Encoded(String s) {
		setData(s.getBytes(UTF8_CHARSET));
	}
	
	/** Do not manipulate the result! */
	public byte[] getData() {
		return data;
	}
	
	@Override
	public Utf8String clone() {
		Utf8String clone = new Utf8String();
		clone.data = this.data.clone();
		return clone;
	}
	
	@Override
	public int hashCode() {
		return Arrays.hashCode(data) + 1;
	}
	
	@Override
	public boolean equals(Object obj) {
		if (obj == this) {
			return true;
		} else if (obj == null || !(obj instanceof Utf8String)) {
			return false;
		}
		return Arrays.equals(((Utf8String) obj).data, data);
	}
	
	@Override
	public int compareTo(Utf8String other) {
		return BYTE_ARRAY_COMPARATOR.compare(this.data, other.data);
	}

	@Override
	public String toString() {
		return "ByteString["+toString(data)+"]";
	}
	
	public static class HashChachingByteString extends Utf8String {
		
		private int hashCode;

		@Override
		public int hashCode() {
			return hashCode;
		}
		
		@Override
		public void setData(byte[] data) {
			super.setData(data);
			hashCode = super.hashCode();
		}
		
	}
}
