package de.uni_potsdam.hpi.isg.utils.db;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import de.uni_potsdam.hpi.isg.utils.datatypes.Utf8String;
import de.uni_potsdam.hpi.isg.utils.db.metamodel.SchemaExtractor;

public class MySqlDatabase extends Database<MySqlDatabase> {

	public MySqlDatabase(Properties properties) throws SQLException {
		super(properties);
	}

	@Override
	public void executeWithoutDb(String command) throws SQLException {
		try (Connection tempConnection = createConnection("")) {
			Statement statement = tempConnection.createStatement();
			statement.execute(command);
			statement.close();
		}
	}

	@Override
	public Connection createConnection() throws SQLException {
		return createConnection(properties.getProperty(PROP_DB_NAME));
	}

	private Connection createConnection(String dbName) throws SQLException {
		try {
			Class.forName("com.mysql.jdbc.Driver");
		} catch (ClassNotFoundException e) {
			throw new RuntimeException(e);
		}
		return DriverManager.getConnection("jdbc:mysql://localhost/" + dbName, properties);
	}

	@Override
	public List<String> findTables() throws SQLException {
		Statement statement = connection.createStatement();
		String request = "show tables in " + properties.getProperty(PROP_DB_NAME) + ";";
		System.out.println(request);
		ResultSet resultSet = statement.executeQuery(request);
		List<String> tablesList = new ArrayList<String>();

		while (resultSet.next()) {
			tablesList.add(resultSet.getString(1));
		}
		System.out.println("tables: " + tablesList.toString());
		return tablesList;
	}

	@Override
	public SchemaExtractor<MySqlDatabase> getSchemaExtractor() {
		throw new UnsupportedOperationException("There is no schema extractor for MySQL (yet).");
	}

	@Override
	public List<String> findColumns(String table) throws SQLException {

		Statement statement = connection.createStatement();
		String request = "show columns from " + table + ";";
		ResultSet resultSet = statement.executeQuery(request);
		List<String> columnsList = new ArrayList<String>();

		while (resultSet.next()) {
			columnsList.add(resultSet.getString(1));

		}
		System.out.println("table: " + table + ", columns: " + columnsList.toString());
		return columnsList;
	}

	@Override
	public void createSortedTable(String table, String column) throws SQLException {
		Statement statement = connection.createStatement();
		String request = "create table " + table + "_" + column + " select distinct " + column
				+ " from " + table + " order by " + column + ";";
		System.out.println(request);
		statement.execute(request);

	}

	@Override
	public Utf8String readValue(String table, String column, Integer rowNumber) throws SQLException {
		Statement statement = connection.createStatement();
		String request = "select " + column + " from " + table + " limit " + rowNumber + ", 1;";
		System.out.println(request);
		ResultSet resultSet = statement.executeQuery(request);
		if (resultSet.next()) {
			return Utf8String.createUtf8For(resultSet.getString(1));
		} else {
			return null;
		}
	}

	@Override
	public void renameTable(String oldName, String newName) {
		throw new RuntimeException("Das muss die Kati machen! ;P");
	}

}
