package de.uni_potsdam.hpi.isg.utils.db;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import de.uni_potsdam.hpi.isg.utils.datatypes.Utf8String;
import de.uni_potsdam.hpi.isg.utils.db.metamodel.PostgresqlSchemaExtractor;

public class PostgresqlDatabase extends Database<PostgresqlDatabase> {

	public static final String GET_TABLES_SQL_CMD = "SELECT table_name "
			+ "FROM information_schema.tables "
			+ "WHERE table_schema = 'public';";

	public static final String GET_COLUMNS_SQL_CMD = "SELECT column_name "
			+ "FROM information_schema.columns WHERE table_name ='%s';";

	public static final String CONVERT_TABLE_SQL_QUERY = "CREATE TABLE %s "
			+ "AS SELECT DISTINCT %s FROM %s;";

	public static final String READ_ROW_SQL_QUERY = "SELECT %s FROM %s LIMIT 1 OFFSET %d";

	public static final String RENAME_TABLE_SQL_QUERY = "ALTER TABLE %s RENAME TO %s;";
	
	public PostgresqlDatabase(Properties properties) throws SQLException {
		super(properties);
	}
	
	@Override
	public void executeWithoutDb(String command) throws SQLException {
		try (Connection tempConnection = createConnection("template1")) {
			Statement statement = tempConnection.createStatement();
			statement.execute(command);
			statement.close();
		};
	}

	@Override
	public Connection createConnection() throws SQLException {
		return createConnection(properties.getProperty(PROP_DB_NAME));
	}

	private Connection createConnection(String dbName) throws SQLException {
		try {
			Class.forName("org.postgresql.Driver");
		} catch (ClassNotFoundException e) {
			throw new RuntimeException(e);
		}
		return DriverManager.getConnection(
				"jdbc:postgresql:" + dbName,
				properties);
	}

	@Override
	public List<String> findTables() throws SQLException {
		Statement statement = connection.createStatement();
		ResultSet resultSet = statement.executeQuery(GET_TABLES_SQL_CMD);
		List<String> tablesList = new ArrayList<String>();

		while (resultSet.next()) {
			tablesList.add(resultSet.getString(1));
		}
		return tablesList;
	}

	@Override
	public PostgresqlSchemaExtractor getSchemaExtractor() {
		return new PostgresqlSchemaExtractor(this);
	}
	
	@Override
	public List<String> findColumns(String table) throws SQLException {
		Statement statement = connection.createStatement();
		ResultSet resultSet = statement.executeQuery(String.format(
				GET_COLUMNS_SQL_CMD, table));
		List<String> columnsList = new ArrayList<String>();

		while (resultSet.next()) {
			columnsList.add(resultSet.getString(1));

		}
		return columnsList;
	}

	@Override
	public void createSortedTable(String table, String column)
			throws SQLException {
		Statement statement = connection.createStatement();
		String query = String.format(CONVERT_TABLE_SQL_QUERY, table + "_"
				+ column, column, table);
		System.out.println(query);
		statement.execute(query);

	}

	@Override
	public Utf8String readValue(String table, String column, Integer rowNumber) throws SQLException {
		Statement statement = connection.createStatement();
		String request = String.format(READ_ROW_SQL_QUERY, column, table, rowNumber);
		ResultSet resultSet = statement.executeQuery(request);
		if (resultSet.next()) {
			return Utf8String.createUtf8For(resultSet.getString(1));
		}
		else {
			return null;
		}
	}

	@Override
	public void renameTable(String oldName, String newName) throws SQLException {
		String query = String.format(RENAME_TABLE_SQL_QUERY, oldName, newName);
		execute(query);
	}
}
