package de.uni_potsdam.hpi.isg.utils.db.metamodel;

/**
 * Columns can have datatypes. For interoperability, it is recommended to
 * use following datatype identifiers. 
 * <ul>
 * <li><tt>boolean</tt></li>
 * <li><tt>int(precision)</tt></li>
 * <li><tt>char(length)</tt></li>
 * <li><tt>varchar(max length)</tt></li>
 * <li><tt>text</tt></li>
 * <li><tt>float(precision)</tt></li>
 * <li><tt>timestamp</tt></li>
 * </ul>
 * Though, other datatype names are also allowed.
 * 
 * @author Sebastian Kruse
 */
public class Column {

	private String name;
	private String dataType;

	public Column(String name) {
		this(name, "unknown");
	}

	public Column(String columnName, String dataType) {
		if (columnName == null || columnName.isEmpty()) {
			throw new IllegalArgumentException();
		}
		if (dataType == null || dataType.isEmpty()) {
			throw new IllegalArgumentException();
		}

		this.name = columnName;
		this.dataType = dataType;
	}

	public String getName() {
		return name;
	}

	public String getDataType() {
		return dataType;
	}

	@Override
	public String toString() {
		return "Column[name=" + name + ", dataType=" + dataType + "]";
	}

}
