package de.uni_potsdam.hpi.isg.utils.db.metamodel;

import java.util.Arrays;
import java.util.List;
import java.util.Map;

public class ForeignKeyConstraint implements Constraint {

	private List<Column> targetColumns;
	private Table targetTable;
	private List<Column> sourceColumns;
	private Table sourceTable;

	public ForeignKeyConstraint(Table sourceTable, Column[] sourceColumns,
			Table targetTable, Column[] targetColumns) {
		if (sourceTable == null || sourceColumns == null
				|| sourceColumns.length == 0 || targetTable == null
				|| targetColumns == null
				|| sourceColumns.length != targetColumns.length) {
			throw new IllegalArgumentException();
		}
		checkColumnConsistency(sourceTable, sourceColumns);
		checkColumnConsistency(targetTable, targetColumns);

		this.sourceTable = sourceTable;
		this.sourceColumns = Arrays.asList(sourceColumns);
		this.targetTable = targetTable;
		this.targetColumns = Arrays.asList(targetColumns);
	}

	private void checkColumnConsistency(Table table, Column[] columns) {
		for (int i = 0; i < columns.length; i++) {
			Column column = columns[i];
			if (column == null
					|| !column.equals(table.getColumns().get(column.getName())))
				throw new IllegalArgumentException();
		}
	}

	@Override
	public Type getType() {
		return Type.FOREIGN_KEY;
	}

	@Override
	public boolean isApplicableTo(RelationalSchema schema) {
		Map<String, Table> tables = schema.getTables();
		return this.sourceTable.equals(tables.get(this.sourceTable.getName()))
				&& this.targetTable.equals(tables.get(this.targetTable
						.getName()));
	}

	public Table getSourceTable() {
		return sourceTable;
	}

	public List<Column> getSourceColumns() {
		return sourceColumns;
	}

	public Table getTargetTable() {
		return targetTable;
	}

	public List<Column> getTargetColumns() {
		return targetColumns;
	}

	public int getArity() {
		return this.sourceColumns.size();
	}
	
	@Override
	public String toString() {
		return "ForeignKeyConstraint[sourceTable=" + sourceTable.getName()
				+ ", sourceColumns=" + sourceColumns + ", targetTable="
				+ targetTable.getName() + ", targetColumns=" + targetColumns
				+ "]";
	}

}
