package de.uni_potsdam.hpi.isg.utils.db.metamodel;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

public class RelationalSchema {

	private String name;
	private Map<String, Table> tables;
	private Collection<Constraint> constraints;

	public RelationalSchema(String name) {
		if (name == null || name.isEmpty())
			throw new IllegalArgumentException();

		this.name = name;
		this.tables = new HashMap<String, Table>();
		this.constraints = new ArrayList<Constraint>();
	}

	public Table addTable(String tableName) {
		if (tableName == null || tableName.isEmpty() || tables.containsKey(tableName))
			throw new IllegalArgumentException();
		
		Table table = new Table(tableName);
		tables.put(tableName, table);
		
		return table;
	}
	
	public Map<String, Table> getTables() {
		return tables;
	}
	
	@Override
	public String toString() {
		return "RelationalSchema[name=" + name + ", tables=" + tables.keySet()
				+ ", " + constraints.size() + " constraints]";
	}

	public String getName() {
		return name;
	}

	public Collection<Constraint> getConstraints() {
		return constraints;
	}

	public void addConstraint(Constraint constraint) {
		if (constraint == null || !constraint.isApplicableTo(this))
			throw new IllegalArgumentException();
		
		constraints.add(constraint);
	}
	
}
