package de.uni_potsdam.hpi.isg.utils.db.metamodel;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class UniqueConstraint implements Constraint {

	private Table table;
	private List<Column> columns;

	public UniqueConstraint(Table table, Column... columns) {
		if (table == null || columns == null || columns.length == 0) {
			throw new IllegalArgumentException();
		}

		for (Column column : columns) {
			if (column == null
					|| !column.equals(table.getColumns().get(column.getName()))) {
				throw new IllegalArgumentException("Illegal column: " + column);
			}
		}

		this.table = table;
		this.columns = Arrays.asList(columns);

	}

	@Override
	public Type getType() {
		return Type.UNIQUE;
	}

	@Override
	public boolean isApplicableTo(RelationalSchema schema) {
		return table.equals(schema.getTables().get(table.getName()));
	}
	
	public Table getTable() {
		return table;
	}
	
	public List<Column> getColumns() {
		return columns;
	}

	@Override
	public String toString() {
		List<String> columnNames = new ArrayList<>();
		for (Column column : columns) {
			columnNames.add(column.getName());
		}
		return "UniqueConstraint[table=" + table.getName() + ", columns="
				+ columnNames + "]";
	}

}
