package de.uni_potsdam.hpi.isg.utils.files;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Arrays;

/**
 * Writes rows to a TSV file.
 */
public class TsvWriter {
	
	private File targetFile;
	private BufferedWriter writer;

	public TsvWriter(String targetFilePath) {
		this.targetFile = new File(targetFilePath);
	}
	
	public TsvWriter(File targetFile) {
		this.targetFile = targetFile;
	}
	
	/** Deletes the target file. */
	public void clear() {
		targetFile.delete();
	}
	
	public void open() throws IOException {
		writer = new BufferedWriter(new FileWriter(
				targetFile, true), 0x1000000);
	}

	/** Writes the given row to the target file. */
	public void log(String[] values) {
		BufferedWriter writer = this.writer;
		try {
			if (writer == null) 
				writer = new BufferedWriter(new FileWriter(
						targetFile, true));
			String delimiter = "";
			for (String value : values) {
				writer.write(delimiter);
				writer.write(value);
				delimiter = "\t";
			}
			writer.write("\n");
		} catch (IOException e) {
			System.err.println("Could not log result "+Arrays.toString(values));
			e.printStackTrace();
		} finally {
			if (this.writer == null)
				close(writer);
		}
	}

	public void close() {
		if (writer != null)
			close(writer);
		writer = null;
	}
	
	private void close(BufferedWriter writer) {
		if (writer != null) {
			try {
				writer.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

}
