package de.uni_potsdam.hpi.isg.utils.in_memory;

import java.util.Comparator;

/**
 * This class represents an in-memory data column. It can additionally have an inverted index on that column. 
 * 
 * @author Sebastian Kruse
 *
 */
public class IntColumn {

	private int index;
	private int[] values;
	private InvertedIndex invertedIndex;

	public IntColumn(int index, int[] values) {
		this.index = index;
		this.values = values;
	}

	public void calculateInvertedIndex() {
		this.invertedIndex = new InvertedIndex(this);
	}
	
	public InvertedIndex getInvertedIndex() {
		if (invertedIndex == null) {
			calculateInvertedIndex();
		}
		return invertedIndex;
	}

	public int getIndex() {
		return index;
	}

	public int[] getValues() {
		return values;
	}

	public boolean isConstant() {
		return getInvertedIndex().isConstant();
	}

	public double getEntropy() {
		return getInvertedIndex().getEntropy();
	}

	public static Comparator<IntColumn> INDEX_COMPARATOR = new Comparator<IntColumn>() {
		public int compare(IntColumn col1, IntColumn col2) {
			return Integer.compare(col1.index, col2.index);
		}
	};
}
