package de.uni_potsdam.hpi.isg.utils.in_memory;

import it.unimi.dsi.fastutil.ints.Int2ObjectMap;
import it.unimi.dsi.fastutil.ints.Int2ObjectOpenHashMap;
import it.unimi.dsi.fastutil.ints.IntArrayList;
import it.unimi.dsi.fastutil.ints.IntList;

import java.util.ArrayList;
import java.util.List;

public class InvertedIndex {

	private List<IntList> rowsWithSameValues = new ArrayList<IntList>();
	private double entropy;
	private int columnSize, containedRows;

	public InvertedIndex(IntColumn column) {
		Int2ObjectMap<IntList> valuesToRows = new Int2ObjectOpenHashMap<IntList>();
		int[] values = column.getValues();
		columnSize = values.length;
		containedRows = 0;

		// invert row value mapping
		for (int row = 0; row < values.length; row++) {
			int value = values[row];
			IntList rows = valuesToRows.get(value);
			if (rows == null) {
				rows = new IntArrayList();
				valuesToRows.put(value, rows);
			}
			rows.add(row);
		}

		// remove values with only one row and calculate entropy on the fly
		double infoCharacteristic = 0d;
		for (IntList rows : valuesToRows.values()) {

			// remove values that occur only once
			if (rows.size() == 1) {
				for (int row : rows) {
					values[row] = -1;
				}
			} else {
				containedRows += rows.size();
				rowsWithSameValues.add(rows);

				// gather specific entropy data
				double n = rows.size();
				infoCharacteristic += n * Math.log(n);
			}
		}

		entropy = -infoCharacteristic / columnSize + Math.log(columnSize);
	}
	

	public double getEntropy() {
		return entropy;
	}

	public boolean isUnique() {
		return rowsWithSameValues.isEmpty();
	}

	public boolean isConstant() {
		return rowsWithSameValues.size() == 1
				&& rowsWithSameValues.iterator().next().size() == columnSize;
	}

	public int getContainedRows() {
		return containedRows;
	}
	
	public List<IntList> getRowsWithSameValues() {
		return rowsWithSameValues;
	}

}
