package de.uni_potsdam.hpi.isg.utils.sampling;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Random;

public class ReservoirSampler<T> {

	private int sampleSize, observedObjects;
	private Random random;
	private List<T> samples;

	public ReservoirSampler(int sampleSize) {
		this.sampleSize = sampleSize;
		this.observedObjects = 0;
		this.random = new Random();
		this.samples = new ArrayList<>(sampleSize);
	}

	public void observe(T obj) {
		observedObjects++;
		if (samples.size() < sampleSize) {
			samples.add(obj);
		} else {
			int position = random.nextInt(observedObjects);
			if (position < sampleSize) {
				samples.set(position, obj);
			}
		}
	}

	public List<T> getSample() {
		return Collections.unmodifiableList(samples);
	}

}