package de.uni_potsdam.hpi.isg.utils.settings;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import de.uni_potsdam.hpi.isg.utils.db.Database;

public class DatabaseSettings extends AbstractSettings {

	@Property("user")
	public String dbUser;

	@Property("password")
	public String dbPassword;

	@Property(Database.PROP_DB_TYPE)
	public String dbType;

	@Property(Database.PROP_DB_NAME)
	public String dbName;

	public static DatabaseSettings from(String dbUri, String userName, String password) {
		// A better solution would be to only extract the database type here and
		// rather pass on the complete URI to the appropriate Database
		// implementation. Furthermore, in principle, some information might get
		// lost by this parsing step.
		DatabaseSettings settings = new DatabaseSettings();
		settings.dbType = resolveDatabaseType(dbUri);
		settings.dbName = resolveDatabaseName(dbUri, settings.dbType);
		settings.dbUser = userName;
		settings.dbPassword = password;
		return settings;
	}

	private static String resolveDatabaseType(String dbUri) {
		if (dbUri.startsWith("jdbc:postgresql:")) {
			return "postgresql";
		}
		throw new RuntimeException("Cannot map the URI " + dbUri + " to a known database type.");
	}

	private static String resolveDatabaseName(String dbUri, String dbType) {
		switch (dbType) {
		case "postgresql":
			Pattern pattern = Pattern.compile("jdbc:postgresql:([A-Za-z0-9_]+)/?");
			Matcher matcher = pattern.matcher(dbUri);
			if (matcher.matches()) {
				return matcher.group(1);
			}
			pattern = Pattern.compile("jdbc:postgresql://.*?/([A-Za-z0-9_]+)");
			matcher = pattern.matcher(dbUri);
			if (matcher.matches()) {
				return matcher.group(1);
			}
		}
		throw new RuntimeException("Cannot extract the database name from " + dbUri);
	}

}
