package de.uni_potsdam.hpi.isg.utils.util;

import it.unimi.dsi.fastutil.ints.Int2ObjectMap;
import it.unimi.dsi.fastutil.ints.Int2ObjectRBTreeMap;
import it.unimi.dsi.fastutil.ints.Int2ObjectSortedMap;
import it.unimi.dsi.fastutil.objects.Object2IntMap;
import it.unimi.dsi.fastutil.objects.Object2IntMap.Entry;
import it.unimi.dsi.fastutil.objects.Object2IntOpenHashMap;
import it.unimi.dsi.fastutil.objects.ObjectSet;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Comparator;
import java.util.List;

public class Counter<T> {

	private Object2IntMap<T> counts;
	
	public Counter() {
		this.counts = new Object2IntOpenHashMap<T>();
		this.counts.defaultReturnValue(0);
	}
	
	public void count(T element) {
		count(element, 1);
	}
	
	public void count(T element, int delta) {
		int count = counts.getInt(element);
		counts.put(element, count + delta);
	}
	
	public int getCount(T element) {
		return counts.getInt(element);
	}
	
	public Object2IntMap<T> getCounts() {
		return counts;
	}
	
	public Collection<Object2IntMap.Entry<T>> getEntries() {
		return counts.object2IntEntrySet();
	}

	public void reset() {
		counts.clear();
	}
	
	public Int2ObjectMap<List<T>> reverse() {
		Int2ObjectSortedMap<List<T>> result = new Int2ObjectRBTreeMap<>(new Comparator<Integer>() {

			@Override
			public int compare(Integer o1, Integer o2) {
				return -Integer.compare(o2, o1);
			}
		});
		ObjectSet<Entry<T>> entries = counts.object2IntEntrySet();
		for (Object2IntMap.Entry<T> entry : entries) {
			int count = entry.getIntValue();
			List<T> list = result.get(count);
			if (list == null) {
				list = new ArrayList<>();
				result.put(count, list);
			}
			list.add(entry.getKey());
		}
		return result;
	}

	@Override
	public String toString() {
		return "Counter [counts=" + counts + "]";
	}
	
}
