package de.uni_potsdam.hpi.isg.utils.util;

import java.util.Collection;
import java.util.Iterator;

public class MultiLevelIterator<T extends Iterable<E>, E> implements Iterator<E> {

	private Iterator<T> topLevelIterator;
	private Iterator<E> subLevelIterator;
	private E next;

	public MultiLevelIterator(Collection<T> collection) {
		this.topLevelIterator = collection.iterator();
		moveToNextElement();
	}

	private void moveToNextElement() {
		// At first move the top-level iterator until it yields a non-empty sublevel iterator.
		while (this.subLevelIterator == null && this.topLevelIterator.hasNext()) {
			T nextIterable = this.topLevelIterator.next();
			this.subLevelIterator = nextIterable.iterator();
			if (!this.subLevelIterator.hasNext()) {
				this.subLevelIterator = null;
			}
		}
		
		// Now evaluate the sublevel iterator at hand.
		if (this.subLevelIterator != null) {
			this.next = this.subLevelIterator.next();
			if (!this.subLevelIterator.hasNext())
				this.subLevelIterator = null;
		} else {
			this.next = null;
		}
	}

	@Override
	public boolean hasNext() {
		return this.next != null;
	}

	@Override
	public E next() {
		E returnValue = this.next;
		moveToNextElement();
		return returnValue;
	}

	@Override
	public void remove() {
		throw new UnsupportedOperationException();
	}
	
	public Iterable<E> wrapWithIterable() {
		return new Iterable<E>() {

			@Override
			public Iterator<E> iterator() {
				return MultiLevelIterator.this;
			}
		};
	}

}
