package de.uni_potsdam.hpi.isg.utils.util;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

public class Permutations {
	
	/**
	 * Generates all permutations of the given list.
	 * @param list the list to be permuted
	 * @return a list of all permutations
	 */
	public static <T> List<List<T>> generateAllPermutations(List<T> list) {
		if (list == null) {
			throw new IllegalArgumentException();
		}
		List<List<T>> permutations = new ArrayList<>();
		List<T> prefix = new ArrayList<>();
		List<T> openElements = new ArrayList<>(list);
		generateAllPermutationsAux(prefix, openElements, permutations);
		
		return permutations;
	}

	private static <T> void generateAllPermutationsAux(List<T> prefix, List<T> openElements, List<List<T>> permutationStore) {
		if (openElements.isEmpty()) {
			permutationStore.add(new ArrayList<>(prefix));
		} else {
			for (int i = 0; i < openElements.size(); i++) {
				T selectedElement = openElements.remove(0);
				prefix.add(selectedElement);
				generateAllPermutationsAux(prefix, openElements, permutationStore);
				prefix.remove(prefix.size() - 1);
				openElements.add(selectedElement);
			}
		}
	}

	public static <T> Collection<List<T>> powerSet(List<T> originalSet) {
		Collection<List<T>> sets = new ArrayList<List<T>>();
        if (originalSet.isEmpty()) {
            return Collections.singleton(Collections.<T>emptyList());
        }
     
        T head = originalSet.get(0);
        List<T> rest = originalSet.subList(1, originalSet.size());
        for (List<T> set : powerSet(rest)) {
            List<T> newSet = new ArrayList<>();
            newSet.add(head);
            newSet.addAll(set);
            sets.add(newSet);
            sets.add(set);
        }
        return sets;
    }
	
	public static <T> List<List<T>> getAllSublists(List<T> list) {
		List<List<T>> sublists = new ArrayList<>();
		for (int length = 1; length <= list.size(); length++) {
			sublists.addAll(getAllSublists(list, length));
		}
		return sublists;
		
	}
	
	public static <T> List<List<T>> getAllSublists(List<T> list, int length) {
		List<List<T>> sublists = new ArrayList<>();
		List<Boolean> isSubsetTemplate = new ArrayList<>(list.size());
		for (int i = 0; i < list.size(); i++) {
			isSubsetTemplate.add(i < length);
		}
		
		for (List<Boolean> isSubsetPermutation : generateAllPermutations(isSubsetTemplate)) {
			List<T> sublist = new ArrayList<>(length);
			int index = 0;
			for (Boolean isSubset : isSubsetPermutation) {
				if (isSubset) {
					sublist.add(list.get(index));
				}
				index++;
			}
			sublists.add(sublist);
		}
		
		return sublists;
	}
	
}
