package de.uni_potsdam.hpi.isg.utils.util;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import org.junit.Assert;
import org.junit.Test;

public class MultiLevelIteratorTest {

	@Test
	public void testAsIterable() {
		List<List<String>> listOfLists = new ArrayList<>();
		listOfLists.add(Arrays.<String> asList());
		listOfLists.add(Arrays.<String> asList("word"));
		listOfLists.add(Arrays.<String> asList("number", "text"));
		listOfLists.add(Arrays.<String> asList());
		listOfLists.add(Arrays.<String> asList());
		listOfLists.add(Arrays.<String> asList("not", "very", "creative"));
		listOfLists.add(Arrays.<String> asList());
		listOfLists.add(Arrays.<String> asList());

		List<String> iteratedValues = new ArrayList<>();
		for (String value : new MultiLevelIterator<>(listOfLists).wrapWithIterable()) {
			iteratedValues.add(value);
		}
		List<String> expectedValues = Arrays.asList("word", "number", "text", "not", "very",
				"creative");
		
		Assert.assertEquals(expectedValues, iteratedValues);
	}
	
	@Test
	public void testEmptyCollectionsOnly() {
		List<List<String>> listOfLists = new ArrayList<>();
		listOfLists.add(Arrays.<String> asList());
		listOfLists.add(Arrays.<String> asList());
		listOfLists.add(Arrays.<String> asList());
		listOfLists.add(Arrays.<String> asList());
		listOfLists.add(Arrays.<String> asList());
		
		List<String> iteratedValues = new ArrayList<>();
		for (String value : new MultiLevelIterator<>(listOfLists).wrapWithIterable()) {
			iteratedValues.add(value);
		}
		Assert.assertEquals(Collections.emptyList(), iteratedValues);
	}
	
	@Test
	public void testEmptyTopLevel() {
		List<List<String>> listOfLists = Collections.emptyList();
		
		List<String> iteratedValues = new ArrayList<>();
		for (String value : new MultiLevelIterator<>(listOfLists).wrapWithIterable()) {
			iteratedValues.add(value);
		}
		Assert.assertEquals(Collections.emptyList(), iteratedValues);
	}

}
